/****************************************************************************\
     ESOPE, EureCard Systems Operating and Programming Environment

     Filename    : EURECARD.H
     Description : This file contains main cross definitions used by the
                   ESOPE functions
                   Include it by entering    #include "eurecard.h"

     Version     : 3.2.6, June 95             (c)  Euresys s.a. 1995
\****************************************************************************/

#ifndef __EURECARD_H__
#define __EURECARD_H__

#ifdef __cplusplus
extern "C"  {
#endif

/***************************************\
  Multiple compiler adaptation support
\***************************************/

#ifdef _MSC_VER
#ifdef _WIN32
#define __far
#define __huge
#define __pascal
#elif(_MSC_VER < 700)
#define __far    _far
#define __huge   _huge
#define __cdecl  _cdecl
#define __pascal _pascal
#endif
#endif

#ifdef __WATCOMC__
#define __cdecl
#ifdef __FLAT__
#define __far
#define __huge
#endif /* __FLAT__ */
#endif /* __WATCOMC__ */

/****************************************************************************\
  Definition of operation codes
\****************************************************************************/

#define OP_FILL       0
#define OP_AND        1
#define OP_ANDNOT     2
#define OP_NOTAND    11
#define OP_NAND      14
#define OP_OR         8
#define OP_ORNOT      4
#define OP_NOTOR     13
#define OP_NOR        7
#define OP_XOR       10
#define OP_XNOR       5
#define OP_EQV       28
#define OP_ADD       16
#define OP_ADDS      17
#define OP_SUB       18
#define OP_SUBS      19
#define OP_MUL       22
#define OP_MULS      23
#define OP_DIV       24
#define OP_MOD       25
#define OP_MAX       20
#define OP_MIN       21
#define OP_AVG       26
#define OP_SHIFT     27
#define OP_CLEAR      3
#define OP_SET       12
#define OP_NEG       15
#define OP_COPY       0
#define OP_SWAP      29
#define OP_THR       30
#define OP_ABS       31
#define OP_NTHR      32
#define OP_ROT       33
#define OP_INV       34

#define OP_DILATE     8
#define OP_MEDIAN     4
#define OP_ERODE      0

/****************************************************************************\
  Definition of Esope functions error codes
\****************************************************************************/

#define OK                                        0
#define NO_EURECARD_FOUND                        -1

#define LCA_CONFIG_FILE_NOT_FOUND                -2
#define EURETALK_PRG_FILE_NOT_FOUND              -3
#define EURETALK_DIC_FILE_NOT_FOUND              -4
#define EURETALK_LNK_FILE_NOT_FOUND              -5

#define TEST_RAM_PROBLEMS                        -6
#define UNABLE_TO_START_INTERPRETER              -7
#define UNABLE_TO_GET_CONNECTION                 -8

#define NO_EURECARD_CONNECTED                    -9
#define INVALID_DESCRIPTOR_NAME                 -10
#define PARAMETERS_OUT_OF_RANGE                 -11

#define NOT_A_VALID_BLOCK_DESCRIPTOR            -12
#define NOT_A_VALID_VECTOR_DESCRIPTOR           -13
#define NOT_A_VALID_KERNEL_DESCRIPTOR           -14
#define NOT_A_VALID_CFG_DESCRIPTOR              -15
#define NOT_A_VALID_VIN_DESCRIPTOR              -16
#define NOT_A_VALID_SYNC_DESCRIPTOR             -17
#define NOT_A_VALID_TIMING_DESCRIPTOR           -18
#define NOT_A_VALID_OPERATION_CODE              -19
#define NOT_A_VALID_FUNCTION_DESCRIPTOR         -20
#define NOT_A_VALID_BLOCK                       -23
#define NOT_A_VALID_FONT_DESCRIPTOR             -24
#define NOT_A_VALID_PEN_DESCRIPTOR              -25
#define NOT_A_VALID_DESCRIPTOR                  -26
#define NOT_A_VALID_VECTOR                      -27
#define NOT_A_VALID_CBLK_DESCRIPTOR             -28
#define NOT_A_VALID_LCA_DESCRIPTOR              -29

#define ATTEMPT_TO_READ_OUT_OF_DATA             -30
#define ATTEMPT_TO_WRITE_OUT_OF_DATA            -31
#define DOS_MEMORY_ALLOCATION_PROBLEM           -32
#define FILE_NOT_FOUND                          -33
#define TOO_MANY_BITS_PER_PIXEL                 -34
#define EURETALK_ERROR                          -35
#define PROTOCOL_ERROR                          -36
#define FILE_ACCESS_PROBLEMS                    -37
#define UNKNOWN_VECTOR_DATA_TYPE                -38
#define DIFFERENT_VECTOR_DATA_TYPES             -39
#define TOO_MANY_PARTICLES_TO_FIT_IN            -40
#define BAD_FUNCTION_NAME                       -41
#define COMPILATION_PROBLEMS                    -42
#define RESULT_TOO_BIG_TO_FIT_IN_VECTOR         -43
#define PIXEL_ISOLATED                          -44
#define PIXEL_OUT_OF_THE_PERIMETER              -45
#define PIXEL_INSIDE_PERIMETER                  -46
#define WINDOW_LIMITS_REACHED                   -47
#define ILLEGAL_KERNEL_VALUE                    -48
#define DEFAULT_LUT_FILE_NOT_FOUND              -49
#define VFC_INVALID_QUAD_USE                    -50
#define VFC_INVALID_OPERATION                   -51
#define OVRL_ACCESS_UNDER_VFC                   -52
#define OUT_OF_WINDOW                           -53
#define NOT_A_VALID_PARAMETER                   -54
#define DICTIONNARY_FULL                        -55
#define UNIMPLEMENTED_FUNCTION                  -56
#define RGB_NOT_VALID_UNDER_VFC                 -57
#define UNABLE_TO_ALLOCATE_MEMORY               -58
#define READ_ONLY_PARAMETER                     -59
#define UNABLE_TO_DOWNLOAD_LCA_CONFIG           -60

#define MAP_FUNCTIONALITY_NOT_PRESENT           -61
#define VFC_FUNCTIONALITY_NOT_PRESENT           -62
#define HST_FUNCTIONALITY_NOT_PRESENT           -63
#define HPC_FUNCTIONALITY_NOT_PRESENT           -63
#define BIC_FUNCTIONALITY_NOT_PRESENT           -64
#define GLC_FUNCTIONALITY_NOT_PRESENT           -65
#define PXC_FUNCTIONALITY_NOT_PRESENT           -66

#define UNKNOWN_BIC_FEATURE                     -67
#define INVALID_SELECTION_ARGUMENT              -68
#define MALLOC_ERROR_SORT_LIST_TOO_LONG         -69
#define BIC_INVALID_DRAWING_MODE                -70
#define MALLOC_ERROR_TOO_MANY_SEGMENTS          -71
#define MALLOC_ERROR                            -72
#define TOO_MANY_OBJECTS_DETECTED               -73
#define GSP_ANALYSE_UNKNOWN_FEATURE             -74
#define OBJECT_NUMBER_OUT_OF_RANGE              -75
#define TOO_MANY_SEGMENTS_DETECTED              -76
#define FEATURE_DEPENDENCY_ERROR                -77
#define OBJECT_TOO_SMALL_FOR_FEATURE            -78
#define RTC_NOT_PRESENT                         -79
#define RTC2_NOT_PRESENT                        -80
#define TIME_OUT_REACHED                        -81
#define DISK_IMAGE_TOO_WIDE_FOR_BLOCK           -82
#define DISK_IMAGE_DEEPER_THAN_BLOCK            -83
#define DISK_FILE_WAS_TALLER_THAN_BLOCK         -84
#define UNCOMPATIBLE_VISILOG_FORMAT             -85
#define UNCOMPATIBLE_TIFF_FORMAT                -86
#define DEFICIENT_PC_IO_CHANNEL                 -87
#define INVALID_UNDER_CTG_MODE                  -88
#define NOT_A_VALID_MESU_DESCRIPTOR             -90
#define MESU_DESC_MUST_NOT_BE_POINT             -91
#define MESU_FUNCTION_NOT_AUTHORIZED            -92
#define DEFECTIVE_PIXEL_CLOCK                  -100
#define DEFECTIVE_TIMING_GENERATION            -101
#define INVALID_STARTING_COLOR                 -102
#define WRONG_SAFE_MEM_SIZE                    -103
#define WRONG_SAFE_MEM_TYPE                    -104
#define AVAILABLE_ONLY_FOR_SOLO                -105
#define PROGRAM_TOO_LARGE_TO_BE_SAVED          -106
#define OBJECTLIB_FUNCTION_NOT_AUTHORIZED      -107
#define CG4_FUNCTIONALITY_NOT_PRESENT          -108
#define FUNCTION_ABORTED                       -109
#define EVA_OVERRUN_ALARM_DETECTED             -110
#define NOT_A_VALID_OCR_DESCRIPTOR             -111
#define MISSING_OBJECT                         -112
#define SUPERFLUOUS_OBJECT                     -113
#define BROKEN_OBJECT                          -114
#define OBJECT_OVER_THRESHOLD                  -115
#define OBJECT_UNDER_THRESHOLD                 -116
#define OCR_DATABASE_FULL                      -118
#define OCR_FUNCTION_NOT_AUTHORIZED            -120
#define OPV_FUNCTION_NOT_AUTHORIZED            -121
#define JPEG_MODULE_NOT_PRESENT                -122
#define ONGOING_BACKGROUND_ACQUISITION         -123

/* solocom errors */
#define REMOTE_DOWNLOAD_NO_SRAM_FOUND          -124
#define REMOTE_DOWNLOAD_SRAM_SIZE_MISMATCH     -125
#define REMOTE_DOWNLOAD_RECEIVED_UNKNOWN_CHAR  -126
#define REMOTE_DOWNLOAD_INEXISTANT_COM_CHANNEL -127
#define REMOTE_DOWNLOAD_READ_ERROR             -128
#define REMOTE_DOWNLOAD_INVALID_BAUD_RATE      -129

#define OCR_TOO_MANY_CHARACTERS_READ           -130

/* keyboard driver errors */
#define KEYBOARD_INITIALIZATION_ERROR          -131
#define KEYBOARD_TRANSMISSION_ERROR            -132
#define KEYBOARD_DATA_REFUSED                  -133
#define KEYBOARD_RESEND_OVERFLOW               -134
#define KEYBOARD_INCORRECT_RESPONSE            -135
#define KEYBOARD_NOT_INITIALIZED               -136

#define REMOTE_DOWNLOAD_TOO_MUCH_DATA          -137

/****************************************************************************\
  Definition used by descriptor functions
\****************************************************************************/

#define ESOPE_VERSION_NUMBER    328

#define TRUE                      1
#define FALSE                     0
#define NIL                      -1

#define DESC_ID             0xDE000000L
#define MAX_DESC_NUMBER     19

#define MSG_ET          0x01
#define MSG_ANSWER      0x02
#define MSG_C           0x04
#define MSG_ET_C        0x08
#define MSG_PUSH        0x10
#define MSG_POP         0x20
#define MSG_USER        0x40
#define MSG_BREAK       0x80

#define MSG_MAX_CHAR    512
#define MSG_MAX_SHORT   256
#define MSG_MAX_LONG    128

#define PEN_DESC     0
#define LCA_DESC     1
#define BLK_DESC     2
#define VIN_DESC     3
#define SYNC_DESC    4
#define TIM_DESC     5
#define DIO_DESC     6
#define COM_DESC     7
#define VOUT_DESC    8
#define OCR_DESC     9
#define JPEG_DESC   10
#define VECT_DESC   11
#define KRNL_DESC   12
#define CBLK_DESC   14
#define EVA_DESC    15
#define BIC_DESC    16
#define CFG_DESC    17
#define FNT_DESC    18
#define MESU_DESC   19

#define DESC_KEY(descriptor)         (*(descriptor)&0xFF000000L)
#define DESC_NUMBER(descriptor)     ((*(descriptor)&0x00FF0000L)>>16)
#define DESC_MAX_NB_PAR(descriptor) ((*(descriptor)&0x0000FF00L)>>8)
#define DESC_APPLIED(descriptor)     (*(descriptor)&0x00000001L)
#define DESC_PREDEF(descriptor)      (*(descriptor)&0x00000004L)
#define DESC_TIM_TYPE(descriptor)    (*(descriptor)&0x00000008L)
#define DESC_HIDDEN_ST(descriptor)   (*(descriptor)&0x00000030L)
#define DESC_NOT_VALID(desc)        ((DESC_KEY(desc)!=DESC_ID)||(DESC_NUMBER(desc)>MAX_DESC_NUMBER))

#ifndef _TMS340

  #define print_error(fct)  printf("\nFunction : %s\nError : %s\n",#fct,err_tab[-fct]);

#endif

#ifndef _TMS340

  typedef unsigned long DESCRIPTOR;
  extern DESCRIPTOR __far _dsc_ptr[];

  typedef unsigned long FUNCTION ;
  extern FUNCTION __far _fct_ptr[];

#else

  typedef int *DESCRIPTOR;

#endif

#ifdef _TMS340

extern int rot_pitch;
extern int memsizex;           /* Video memory size (512/1024)              */
extern int memsizey;
extern int bufferb;            /* GSP address of video buffer B             */

#define gsp_blk_address(blk)      (((*(blk+BLK_BUF))?bufferb:0)+(*(blk+BLK_ORGX)<<4)+(*(blk+BLK_ORGY)<<rot_pitch))
#define gsp_blk_work_address(blk) (((*(blk+BLK_BUF))?bufferb:0)+(*(blk+BLK_WORK_ORGX)<<4)+(*(blk+BLK_WORK_ORGY)<<rot_pitch))

#endif

#define fix2float(n) (((float)(n))/65536.00f)
#define float2fix(n) ((long)((n)*65536.00f))

/****************************************************************************\
  Definition of descriptors' parameters
\****************************************************************************/

#define BLK_ID                0
#define BLK_ORGX              1
#define BLK_ORGY              2
#define BLK_WIDTH             3
#define BLK_HEIGHT            4
#define BLK_TYPE              5
#define   MEM_BLK          0
#define   ACQ_BLK          1
#define   VISU_BLK         2
#define   OVL_BLK          3
#define   EVA_BLK          4
#define   JPEG_BLK         5
#define   EUREBUS_BLK      6
#define BLK_SELECT            6
#define BLK_BUF               7
#define BLK_SELECT1           8
#define BLK_BUF1              9
#define BLK_SELECT2          10
#define BLK_BUF2             11
#define BLK_MODE             12
#define   BW_BLK           0
#define   COLOR_BLK        1
#define BLK_BINARY           13
#define BLK_HSCALE           14
#define BLK_VSCALE           15
#define BLK_FLIP_HORIZONTAL  16
#define BLK_FLIP_VERTICAL    17
#define BLK_PERMUT_ROW_COL   18
#define BLK_WORK_ORGX        19
#define BLK_WORK_ORGY        20
#define BLK_WORK_WIDTH       21
#define BLK_WORK_HEIGHT      22
#define BLK_LIVE_REFRESH     23

#define KRNL_ID               0
#define KRNL_ALLOCATED_SIZE   1
#define KRNL_DATA_PTR         2
#define KRNL_SIZE_X           3
#define KRNL_SIZE_Y           4
#define KRNL_GAIN             5
#define KRNL_RECTIFIER        6
#define KRNL_OUTSIDE_VALUE    7
#define KRNL_OFFSET           8
#define KRNL_LUT              9

#define VECT_ID               0
#define VECT_ALLOCATED_SIZE   1
#define VECT_DATA_PTR         2
#define VECT_DATA_TYPE        3
#define VECT_N_GROUP          4
#define VECT_GROUP_LENGTH     5
#define VECT_PARAM0           6
#define VECT_PARAM1           7
#define VECT_PARAM2           8
#define VECT_PARAM3           9
#define VECT_PARAM4          10
#define VECT_PARAM5          11
#define VECT_PARAM6          12
#define VECT_PARAM7          13
#define VECT_SIGN            14
#define VECT_AUTO_CLEAR      15
#define VECT_USE             16

#define VECT_USE_SEGMENT      1
#define VECT_USE_TRANS_LIST   2
#define VECT_USE_EXTREM_LIST  3
#define VECT_USE_CONTOUR      4
#define VECT_USE_TRANS_SCAN   5
#define VECT_USE_HIST         6
#define VECT_USE_ROW_PROJ     7
#define VECT_USE_COL_PROJ     8
#define VECT_USE_LUT          9
#define VECT_USE_MESU        10
#define VECT_USE_CG4         11
#define VECT_USE_OCR         12
#define VECT_USE_COMPARE     13
#define VECT_USE_JPEG        14

#define PEN_ID                0
#define PEN_INK_COLOR         1
#define PEN_BACK_COLOR        2
#define PEN_OUTLINE_COLOR     3
#define PEN_DIRECTION         4
#define PEN_CLIPPING_MODE     5
#define PEN_OPERATION         6

#define FNT_ID                0
#define FNT_ALLOCATED_SIZE    1
#define FNT_DATA_PTR          2
#define FNT_X_SIZE            3
#define FNT_Y_SIZE            4
#define FNT_FIRST_CHAR        5
#define FNT_LAST_CHAR         6
#define FNT_CHAR_LEN          7

#define VIN_ID                   0
#define VIN_CHANNEL              1
#define VIN_POLARITY             2
#define VIN_FILTER               3
#define VIN_GAIN                 4
#define VIN_OFFSET               5
#define VIN_LUT                  6
#define VIN_SYNC_SOURCE          7
#define VIN_SYNC_DETECT_LEVEL    8
#define VIN_GRAB_SYNC            9
#define VIN_GRAB_PAUSE          10
#define VIN_GRAB_DELAY          11
#define VIN_GRAB_DURATION       12
#define VIN_GRAB_STROBE         13
#define VIN_SHUTTER             14
#define VIN_DUMP_ENABLE         15
#define VIN_CID_WINDOW          16
#define VIN_CID_MEM_TIMING      17
#define VIN_VCR                 18
#define VIN_GRAB_STROBE_LENGTH  19
#define VIN_ASYNC_SHUTTER       20
#define VIN_ASYNC_INTEGRATION   21
#define VIN_GRAB_MODE           22
#define   VIN_GRAB_MODE_NORMAL      0
#define   VIN_GRAB_MODE_CONTINUOUS  1
#define   VIN_GRAB_MODE_BACKGROUND  2
#define VIN_ONGOING_ACQUISITION 23

#define VIN_DESC_SIZE           24

#define SYNC_ID               0
#define SYNC_CHANNEL          1
#define SYNC_POLARITY         2
#define SYNC_HOR              3
#define SYNC_VERT             4
#define SYNC_VDET             5

#define TIM_ID                0
#define TIM_ALLOCATED_SIZE    1
#define TIM_DATA_PTR          2
#define TIM_DELAYX            3
#define TIM_DELAYY            4
#define TIM_DIMX              5
#define TIM_DIMY              6
#define TIM_POSX              7
#define TIM_POSY              8
#define TIM_SCANX             9
#define TIM_SCANY            10
#define TIM_ORGX             11
#define TIM_ORGY             12
#define TIM_SIZEX            13
#define TIM_SIZEY            14
#define TIM_OFFSETX          15
#define TIM_OFFSETY          16
#define TIM_FREQ             17
#define TIM_RANGE            18
#define TIM_HFAC             19
#define TIM_VFAC             20
#define TIM_CR               21
#define TIM_NINTRL           22
#define TIM_NUMBER           23
#define TIM_HTOTAL           24
#define TIM_VTOTAL           25
#define TIM_H_POLARITY       26
#define TIM_V_POLARITY       27
#define TIM_CLOCK_POLARITY   28
#define TIM_SYNC_SOURCE      29
#define TIM_ADJUST_H         30
#define TIM_ADJUST_V         31

#define TIM_DESC_SIZE        32

#define DIO_ID                0
#define DIO_Q1                1
#define DIO_Q2                2
#define DIO_Q3                3
#define DIO_Q4                4
#define DIO_Q5                5
#define DIO_Q6                6
#define DIO_Q7                7
#define DIO_Q8                8
#define DIO_Q9                9
#define DIO_Q10              10
#define DIO_STROBE           11
#define DIO_BUSY             12
#define DIO_PASS_FAIL        13
#define DIO_DV               14
#define DIO_DH               15
#define DIO_D1               16
#define DIO_D2               17
#define DIO_D3               18
#define DIO_D4               19
#define DIO_D5               20
#define DIO_D6               21
#define DIO_TRIGGER          22
#define DIO_SENSE0           23
#define DIO_SENSE1           24
#define DIO_SENSE2           25
#define DIO_SENSE3           26
#define DIO_THR              27
#define DIO_LOCK             28
#define DIO_H                29
#define DIO_V                30
#define DIO_DETS             31
#define DIO_DETV             32
#define DIO_VCO              33
#define DIO_RED_LED          34
#define DIO_GREEN_LED        35
#define DIO_WATCHDOG         36
#define DIO_SIGNAL_WATCHDOG  37
#define DIO_TIME_OUT         38

#define DIO_DESC_SIZE        39

#define COM_ID                0
#define COM_D0                1
#define COM_D1                2
#define COM_D2                3
#define COM_D3                4
#define COM_D4                5
#define COM_D5                6
#define COM_D6                7
#define COM_D7                8
#define COM_D7_0              9
#define COM_Q0               10
#define COM_Q1               11
#define COM_Q2               12
#define COM_Q3               13
#define COM_Q4               14
#define COM_Q5               15
#define COM_Q6               16
#define COM_Q7               17
#define COM_Q7_0             18
#define COM_QMASK            19

#define COM_RXA_ENABLE         20
#define COM_TXA_ENABLE         21
#define COM_RXA_BUFFER_LENGTH  22
#define COM_TXA_BUFFER_LENGTH  23
#define COM_A_PARITY_BIT       24
#define COM_A_STOP_BIT         25
#define COM_RXA_CHAR_LENGTH    26
#define COM_TXA_CHAR_LENGTH    27
#define COM_A_BAUD_RATE        28
#define COM_RXA_N_CHAR         29
#define COM_TXA_N_CHAR         30
#define COM_A_ERROR            31
#define COM_A_RTS              32
#define COM_A_DTR              33
#define COM_A_CTS              34
#define COM_A_DCD              35
#define COM_A_RI               36
#define COM_A_DSR              37
#define COM_A_AUTO_ENABLE      38

#define COM_RXB_ENABLE         39
#define COM_TXB_ENABLE         40
#define COM_RXB_BUFFER_LENGTH  41
#define COM_TXB_BUFFER_LENGTH  42
#define COM_B_PARITY_BIT       43
#define COM_B_STOP_BIT         44
#define COM_RXB_CHAR_LENGTH    45
#define COM_TXB_CHAR_LENGTH    46
#define COM_B_BAUD_RATE        47
#define COM_RXB_N_CHAR         48
#define COM_TXB_N_CHAR         49
#define COM_B_ERROR            50
#define COM_B_RTS              51
#define COM_B_DTR              52
#define COM_B_CTS              53
#define COM_B_DCD              54
#define COM_B_RI               55
#define COM_B_DSR              56
#define COM_B_AUTO_ENABLE      57

#define COM_RX_BUFFER_OVERFLOW  1
#define COM_RX_PARITY_ERROR     2

#define COM_CHANNEL_A           1
#define COM_CHANNEL_B           0

#define VOUT_ID               0
#define VOUT_LUT              1
#define VOUT_LUT_NUMBER       2
#define VOUT_FILTER           3
#define VOUT_RED_L            4
#define VOUT_RED_C            5
#define VOUT_RED_N            6
#define VOUT_RED_B            7
#define VOUT_RED_LSD          8
#define VOUT_RED_NSD          9
#define VOUT_RED_BSD         10
#define VOUT_GRN_L           11
#define VOUT_GRN_C           12
#define VOUT_GRN_N           13
#define VOUT_GRN_B           14
#define VOUT_GRN_LSD         15
#define VOUT_GRN_NSD         16
#define VOUT_GRN_BSD         17
#define VOUT_BLU_L           18
#define VOUT_BLU_C           19
#define VOUT_BLU_N           20
#define VOUT_BLU_B           21
#define VOUT_BLU_LSD         22
#define VOUT_BLU_NSD         23
#define VOUT_BLU_BSD         24
#define VOUT_COLOR0          25
#define VOUT_COLOR1          26
#define VOUT_COLOR2          27
#define VOUT_COLOR3          28
#define VOUT_COLOR4          29
#define VOUT_COLOR5          30
#define VOUT_COLOR6          31
#define VOUT_COLOR7          32
#define VOUT_COLOR8          33
#define VOUT_COLOR9          34
#define VOUT_COLOR10         35
#define VOUT_COLOR11         36
#define VOUT_COLOR12         37
#define VOUT_COLOR13         38
#define VOUT_COLOR14         39
#define VOUT_COLOR15         40
#define VOUT_LUT0            41
#define VOUT_LUT1            42
#define VOUT_LUT2            43
#define VOUT_LUT3            44
#define VOUT_LUT4            45
#define VOUT_LUT5            46
#define VOUT_LUT6            47
#define VOUT_LUT7            48
#define VOUT_LUT8            49
#define VOUT_LUT9            50
#define VOUT_LUT10           51
#define VOUT_LUT11           52
#define VOUT_LUT12           53
#define VOUT_LUT13           54
#define VOUT_LUT14           55
#define VOUT_LUT15           56
#define VOUT_COMPOSITE_SYNC  57
#define VOUT_SYNC1_POL       58
#define VOUT_SYNC1_MODE      59
#define VOUT_SYNC2_POL       60
#define VOUT_SYNC2_MODE      61
#define VOUT_VGA_KEY         62
#define VOUT_VGA_MASK        63
#define VOUT_VGA_MIXER       64
#define VOUT_VGA_CLEAR       65
#define VOUT_VGA_BYPASS      66
#define VOUT_VGA_CTRL_TABLE  67

#define gsp_to_color(Red,Green,Blue,Alpha) (long)((((long)(Red))<<24L)+(((long)(Green))<<16L)+(((long)(Blue))<<8L)+((long)Alpha))

#define LCA_ID               0
#define LCA_ALLOCATED_SIZE   1
#define LCA_DATA_PTR         2
#define LCA_NATURE           3
#define LCA_VERSION          4

#define CBLK_ID                   0
#define CBLK_SEG_ALLOCATED_SIZE   1
#define CBLK_SEG_DATA_PTR         2
#define CBLK_OBJ_ALLOCATED_SIZE   3
#define CBLK_OBJ_DATA_PTR         4
#define CBLK_N_SEG                5
#define CBLK_SEG_SIZE             6
#define CBLK_N_OBJ                7
#define CBLK_OBJ_SIZE             8
#define CBLK_SELECT_LINK_START    9
#define CBLK_SELECT_LINK_LENGTH  10
#define CBLK_ORGX                11
#define CBLK_ORGY                12
#define CBLK_SOURCE_BLK          13
#define CBLK_CONTINUOUS          14
#define CBLK_CUT_OBJECTS         15
#define CBLK_Y_ORIGIN_OFFSET     16
#define CBLK_MAX_OBJECT_HEIGHT   17

#define OCR_ID                          0
#define OCR_TEXT_COLOR                  1
#define    OCR_TEXT_B_ON_W           0
#define    OCR_TEXT_W_ON_B           1
#define OCR_MAX_TEXT_LEN                2
#define OCR_IMAGE_WIDTH                 3
#define OCR_TEXT_LEN                    4
#define OCR_MIN_CHAR_WIDTH              5
#define OCR_MAX_CHAR_WIDTH              6
#define OCR_MIN_CHAR_HEIGHT             7
#define OCR_MAX_CHAR_HEIGHT             8
#define OCR_CHAR_CLASS_PTR              9
#define OCR_REBUILD_CUT_CHAR           10
#define OCR_PROJ_ROW_THR               11
#define OCR_PROJ_COL_THR               12
#define OCR_BAND_HEIGHT_TOLERANCE      13
#define OCR_WORKMEM_PTR                14
#define OCR_TEXT_DATA_PTR              15
#define OCR_DILATE_SKIPPED_CHARACTERS  16
#define OCR_DESIRED_TEXT_LEN           17
#define OCR_MAX_NUM_DILATIONS          18
#define OCR_DILATION_STEP_SIZE         19

#define OCR_VECT_MAGIC            VECT_PARAM0
#define OCR_MAX_CHAR_DATABASE     VECT_PARAM1
#define OCR_NUM_CHAR_DATABASE     VECT_PARAM2
#define OCR_AVG_CHAR_WIDTH        VECT_PARAM3
#define OCR_AVG_CHAR_HEIGHT       VECT_PARAM4
#define OCR_AVG_CHAR_SPACE        VECT_PARAM5

/* Read text : structure of a character element */
#define OCR_TEXT_ASCII_CODE     0
#define OCR_TEXT_CONFIDENCE     1
#define OCR_TEXT_SCORE          2
#define OCR_TEXT_WIDTH          3
#define OCR_TEXT_HEIGHT         4
#define OCR_TEXT_CENTER_X       5
#define OCR_TEXT_CENTER_Y       6

#define OCR_TEXT_ELEMENT_SIZE   7


#define JPEG_ID                   0
#define JPEG_STORAGE_MODE         1
#define  JPEG_STORAGE_RGB       0
#define  JPEG_STORAGE_Y         1
#define  JPEG_STORAGE_Y_COMPACT 2
#define JPEG_COMPRESSION_MODE     2
#define  JPEG_COMPRESS_RGB      0
#define  JPEG_COMPRESS_Y        1
#define JPEG_QUALITY              3
#define JPEG_EOI_MARKER           4
#define JPEG_ACQUISITION          5
#define JPEG_Y_PLANE              6
#define  JPEG_R_PLANE           1
#define  JPEG_G_PLANE           2
#define  JPEG_B_PLANE           4
#define  JPEG_X_PLANE           8

#define JPEG_DESC_SIZE            7

#define EVA_ID                    0
#define EVA_BUFFER_SYNC           1
#define EVA_MAX_LIMIT             2
#define EVA_CCD_LENGTH            3
#define EVA_IDC_MODE              4
#define EVA_INTERRUPT             5
#define EVA_Q0                    6
#define EVA_Q1                    7
#define EVA_Q2                    8
#define EVA_Q3                    9
#define EVA_POL_TRIG             10
#define EVA_EXT_EXPOSE           11
#define EVA_HS_TRIG              12
#define EVA_ONE_SHOT             13
#define EVA_CLOCK                14
#define EVA_PIXEL_PHASE          15
#define EVA_SCALE                16
#define EVA_EXPOSE               17
#define EVA_PERIOD               18
#define EVA_PITCHX               19
#define EVA_PITCHY               20
#define EVA_INIT_ADDRESS         21
#define EVA_D0                   22
#define EVA_D1                   23
#define EVA_FULL                 24
#define EVA_STATE                25
#define EVA_BUFFER               26
#define EVA_IDENT                27
#define EVA_SOFT_TRIG            28
#define EVA_BUFFER_LINES         29
#define EVA_PAGE_DELAY           30
#define EVA_SOFT_PAGE_TRIG       31
#define EVA_HS_PAGE_TRIG         32
#define EVA_POL_PAGE_TRIG        33
#define EVA_CCD_START            34
#define EVA_LINE_START           35
#define EVA_CLG1_BIN             36
#define EVA_COLOR_SHIFT          37
#define EVA_OVERRUN_DETECTOR     38
#define EVA_OVERRUN_ALARM        39
#define EVA_DUAL_CHANNEL         40
#define EVA_HALF_PROFILE         41

#define EVA_1                     0
#define EVA_2                     1
#define EVA_CLCX_1                2
#define EVA_CLCX_2                3
#define EVA_CLG1_GM               4
#define EVA_CLG1_RB               5
#define EVA_IDC_1                 6
#define EVA_IDC_2                 7
#define EVA_TDI_1                 8
#define EVA_TDI_2                 9
#define EVA_IDC_3                10
#define EVA_IDC_4                11

#define MAX_EVA_NUMBER           12

#define EVA_SIMPLE_PROFILE        1
#define EVA_LOW_PROFILE           2
#define EVA_HIGH_PROFILE          3

#define BIC_ID                    0
#define BIC_WHITE_CLASS           1
#define BIC_NEUTRAL_CLASS         2
#define BIC_BLACK_CLASS           3
#define BIC_CODE_TIME_OUT         4
#define BIC_RECONSTRUCT_TIME_OUT  5

#define INSERT_ALL          0
#define INSERT_MIN          1
#define INSERT_MAX          2
#define INSERT_RANGE        3
#define REMOVE_ALL          4
#define REMOVE_MIN          5
#define REMOVE_MAX          6
#define REMOVE_RANGE        7

#define INSERT_IN           8
#define INSERT_TOUCH        9
#define INSERT_OUT         10
#define REMOVE_IN          11
#define REMOVE_TOUCH       12
#define REMOVE_OUT         13
#define REMOVE_BORDER      14

#define SORT_ASCENDING     15
#define SORT_DESCENDING    16

#define OBJ_SEG_NUMBER          0
#define OBJ_SEG_PTR             1
#define OBJ_SELECT_LINK         2
#define OBJ_CLASS               3
#define OBJ_FEATURES            4
#define OBJ_AREA                5
#define OBJ_LARGEST_SEGMENT     6
#define OBJ_GRAVITY_CENTER_X    7
#define OBJ_GRAVITY_CENTER_Y    8
#define OBJ_LIMIT_CENTER_X      9
#define OBJ_LIMIT_CENTER_Y     10
#define OBJ_LIMIT_WIDTH        11
#define OBJ_LIMIT_HEIGHT       12
#define OBJ_LIMIT45_CENTER_X   13
#define OBJ_LIMIT45_CENTER_Y   14
#define OBJ_LIMIT45_WIDTH      15
#define OBJ_LIMIT45_HEIGHT     16
#define OBJ_CONTOUR_X          17
#define OBJ_CONTOUR_Y          18
#define OBJ_PIXEL_MIN          19
#define OBJ_PIXEL_MAX          20
#define OBJ_SIGMA_X            21
#define OBJ_SIGMA_Y            22
#define OBJ_SIGMA_XY           23
#define OBJ_SIGMA_XX           24
#define OBJ_SIGMA_YY           25
#define OBJ_ELLIPSE_WIDTH      26
#define OBJ_ELLIPSE_HEIGHT     27
#define OBJ_ELLIPSE_ANGLE      28
#define OBJ_CENTROID_X         29
#define OBJ_CENTROID_Y         30

#define OCR_MIN_OBJ_SIZE       OBJ_LIMIT45_CENTER_X


#define FTR_SEG_NUMBER                0x1L
#define FTR_SEG_PTR                   0x2L
#define FTR_SELECT_LINK               0x4L
#define FTR_CLASS                     0x8L

#define FTR_AREA                     0x10L
#define FTR_FEATURES                 0x20L

#define FTR_GRAVITY_CENTER           0xC0L
#define FTR_GRAVITY_CENTER_X         0x40L
#define FTR_GRAVITY_CENTER_Y         0x80L

#define FTR_LIMIT                   0xF00L
#define FTR_LIMIT_CENTER_X          0x100L
#define FTR_LIMIT_CENTER_Y          0x200L
#define FTR_LIMIT_WIDTH             0x400L
#define FTR_LIMIT_HEIGHT            0x800L

#define FTR_LIMIT45                0xF000L
#define FTR_LIMIT45_CENTER_X       0x1000L
#define FTR_LIMIT45_CENTER_Y       0x2000L
#define FTR_LIMIT45_WIDTH          0x4000L
#define FTR_LIMIT45_HEIGHT         0x8000L

#define FTR_SIGMA                0x1F0000L
#define FTR_SIGMA_X               0x10000L
#define FTR_SIGMA_Y               0x20000L
#define FTR_SIGMA_XY              0x40000L
#define FTR_SIGMA_XX              0x80000L
#define FTR_SIGMA_YY             0x100000L

#define FTR_ELLIPSE              0xE00000L
#define FTR_ELLIPSE_WIDTH        0x200000L
#define FTR_ELLIPSE_HEIGHT       0x400000L
#define FTR_ELLIPSE_ANGLE        0x800000L

#define FTR_LARGEST_SEGMENT     0x1000000L

#define FTR_CONTOUR             0x6000000L
#define FTR_CONTOUR_X           0x2000000L
#define FTR_CONTOUR_Y           0x4000000L

#define FTR_CENTROID           0x18000000L
#define FTR_CENTROID_X         0x08000000L
#define FTR_CENTROID_Y         0x10000000L

#define FTR_PIXEL_MIN_MAX      0x60000000L
#define FTR_PIXEL_MIN          0x20000000L
#define FTR_PIXEL_MAX          0x40000000L

#define FTR_PIXEL_AVERAGE          3
#define FTR_PIXEL_STD_DEVIATION    4
#define FTR_PIXEL_COUNT            5
#define FTR_PIXEL_THR              6
#define FTR_HIST_MIN               7
#define FTR_HIST_MAX               8
#define FTR_PIXEL_HIST_MIN         9
#define FTR_PIXEL_HIST_MAX        10

#define SEG_NUM                    0
#define SEG_X                      1
#define SEG_Y                      2
#define SEG_LEN                    3
#define SEG_CLASS                  4
#define SEG_POINTER                5
#define SEG_SIZE                   8

#define R_VECT_REF                 0
#define R_VECT_INSPECT             1
#define R_VECT_NEXT                2
#define R_VECT_MULTI               3    /* 0 : not in chain , 1 if chained */
#define R_VECT_ERROR               4
#define R_VECT_FEATURE             5
#define R_VECT_GROUP_SIZE          6

#define COMPARE_ABSOLUTE           1
#define COMPARE_RELATIVE           2

#define CFG_ID                     0
#define CFG_EURECARD               1
#define   EURECARD_BRIO         1
#define   EURECARD_SOLO         2
#define   EURECARD_PRIMO        3
#define CFG_KEY_NUMBER             2
#define CFG_INPUT                  3
#define   INPUT_PRESENT         1
#define CFG_TIMER                  4
#define   PTS_PRESENT           1
#define   CTG_PRESENT           2
#define CFG_MEM                    5
#define   MEM_1024_PRESENT      1
#define CFG_MAPPER                 6
#define   MAP_PRESENT           1
#define   VFC_PRESENT           2
#define CFG_EXTENSION              7
#define   RTC_PRESENT           1
#define   RTC2_PRESENT          2
#define   JPEG_PRESENT          3
#define CFG_CALCULATOR             8
#define   PXC_PRESENT           1
#define   HST_PRESENT           2
#define   HPC_PRESENT           2
#define   BIC_PRESENT           3
#define   GLC_PRESENT           4
#define   CG4_PRESENT           5
#define   MMC_PRESENT           6
#define   PXC_PENDING          11
#define   HPC_PENDING          12
#define   BIC_PENDING          13
#define   MMC_PENDING          16
#define CFG_EVA                    9
#define CFG_EUREBOX               10
#define   EB2_STD_PRESENT       1
#define   EB2_TM6_7_PRESENT     2
#define   EB2_CID2250_PRESENT   3
#define   EB2_XC75_PRESENT      4
#define CFG_CAMERA                11
#define   STD_CAMERA            1
#define   TM6_7_CAMERA          2
#define   CID2250_CAMERA        3
#define   XC75_CAMERA           4
#define   VHR2000_CAMERA        5
#define CFG_MAX_MEM               12
#define CFG_FREE_MEM              13
#define CFG_ESOPE_REVISION        14
#define CFG_PRIMO_VGA             15
#define CFG_PCB_REVISION          16
#define CFG_OBJECTLIB_KEY         17
#define CFG_MESULIB_KEY           18
#define CFG_SOLOCOM               19
#define   SOLOCOM_PRESENT       1
#define CFG_OCRLIB_KEY            20
#define CFG_OPVLIB_KEY            21

#define SAFE_MEM_512K         1
#define SAFE_MEM_1M           2
#define SAFE_MEM_EPROM        3
#define SAFE_MEM_SRAM         4
#define SAFE_MEM_256K         5

#define FUNCTIONALITY_PXC     1
#define FUNCTIONALITY_HST     2
#define FUNCTIONALITY_HPC     2
#define FUNCTIONALITY_BIC     3
#define FUNCTIONALITY_GLC     4
#define FUNCTIONALITY_MAP     5
#define FUNCTIONALITY_VFC     6
#define FUNCTIONALITY_PTS     7
#define FUNCTIONALITY_CTG     8
#define FUNCTIONALITY_MMC     9

#define START_OF_FIELD_EVENT           1
#define START_OF_EVEN_FIELD_EVENT      2
#define START_OF_ODD_FIELD_EVENT       3
#define TRIGGER_EVENT                  4

#define MESU_ID                                0
#define MESU_ACTIVITY                          1
#define MESU_TRANSITION                        2
#define      MESU_BW_OR_WB                  0
#define      MESU_BW                        1
#define      MESU_WB                        2
#define      MESU_BWB                       3
#define      MESU_WBW                       4
#define MESU_INVERSE_TRANSITION                3
#define MESU_TRANSITION_SMOOTHING              4
#define MESU_DERIVATIVE_THRESHOLD              5
#define MESU_TRANSITION_CHOICE                 6
#define      MESU_BEST_SHARPNESS            0
#define      MESU_CENTRE                    1
#define      MESU_FROM_BEGIN                2
#define      MESU_FROM_END                  3
#define MESU_TRANSITION_HYSTERESIS             7
#define MESU_TRANSITION_MIN_GRAY_STEP          8
#define MESU_RESERVED_4                        9
#define MESU_POINT_POSITION_X                 10
#define MESU_POINT_POSITION_Y                 11
#define MESU_POINT_TOLERANCE                  12
#define MESU_POINT_TOLERANCE_VERTICALLY       13
#define MESU_RATIO_X_BY_Y                     14
#define MESU_TYPE                             15
#define      MESU_POINT                     0
#define      MESU_LINE                      1
#define      MESU_PARABOLA                  2
#define      MESU_CIRCLE                    3
#define MESU_OUTLINE_TOLERANCE                16
#define MESU_SAMPLING_STEP                    17
#define MESU_FILTERING_THRESHOLD              18
#define MESU_FILTERING_COUNT                  19
#define MESU_NOMINAL_X                        20
#define MESU_NOMINAL_Y                        21
#define MESU_NOMINAL_RADIUS                   22
#define MESU_NOMINAL_SLOPE                    23
#define MESU_FIXED_SLOPE                      24
#define MESU_BASE_VERTICALLY                  25
#define MESU_BASE_SIZE                        26
#define MESU_SECTOR_AXE                       27
#define MESU_SECTOR_EXCURSION                 28
#define MESU_CIRCLE_CENTRING                  29
#define MESU_CIRCLE_ISOTROPIC                 30
#define MESU_RESERVED_3                       31
#define MESU_RESERVED_2                       32
#define MESU_CIRCLE_ITERATION_COUNT           33
#define MESU_CIRCLE_MAX_DISPLACEMENT          34
#define MESU_POINT_COUNT                      35
#define MESU_POINT_VALID_COUNT                36
#define MESU_MEASURED_X                       37
#define MESU_MEASURED_Y                       38
#define MESU_MEASURED_RADIUS                  39
#define MESU_MEASURED_SLOPE                   40
#define MESU_MEASURED_STD_DEVIATION           41
#define MESU_TRANSITION_NUMBER                42
#define MESU_DETECT_ONLY                      43
#define MESU_POINT_SLOPE                      44
#define MESU_CIRCLE_NORMAL_POINT              45
#define      MESU_CIRCLE_SAMPLE_R_TETA      3
#define MESU_RESERVED_1                       46
#define MESU_RESERVED_0                       47

#define MESU_SIZE                             48

#define MESU_PREFERENCE_CENTRE                 6
#define MESU_PREFERENCE_HYSTERESIS             7
#define MESU_TRANSITION_MIN_AREA               8
#define MESU_POINT_SELECTION                   9
#define MESU_CIRCLE_SAMPLING_STEP             31
#define MESU_CIRCLE_AUTOMATIC_STOP            32

#define DEF_FULL_CCIR_TIM             1
#define DEF_FULL_EIA_TIM              2
#define DEF_SQUARE_CCIR_TIM           3
#define DEF_SQUARE_EIA_TIM            4
#define DEF_512_SQUARE_CCIR_TIM       5
#define DEF_512_SQUARE_EIA_TIM        6

#define DRAW_CBLK_MAX_VALUE           0
#define DRAW_CBLK_LABELISE_UP         1
#define DRAW_CBLK_LABELISE_DOWN       2
#define DRAW_CBLK_CLASS               3
#define DRAW_CBLK_LABELISE_UP_WRAP    4
#define DRAW_CBLK_LABELISE_DOWN_WRAP  5

#define DRAW_CBLK_OBJECTS_IN_BLK  0x10000
#define DRAW_CBLK_ALL_OBJECTS     0


#define INTERRUPT_VECTORS_BYTES      128L
#define ADDRESS_OF_INTERRUPT_VECTORS 0xFFFFFC00L


#ifdef _TMS340

/****************************************************************************\
  EureTalk Data Stack manipulation macros
\****************************************************************************/

extern int *dsp ;                 /* Data stack pointer         */
extern int *rsp ;                 /* Return stack pointer       */

#define pushd(x) *--dsp=x
#define pushr(x) *--rsp=x
#define popd()   *dsp++
#define popr()   *rsp++

#define EDM_unsigned_char_from_stack()  ((unsigned char)*dsp++)
#define EDM_char_from_stack()           ((char)*dsp++)
#define EDM_unsigned_short_from_stack() ((unsigned short)*dsp++)
#define EDM_short_from_stack()          ((short)*dsp++)
#define EDM_unsigned_int_from_stack()   ((unsigned int)*dsp++)
#define EDM_int_from_stack()            ((int)*dsp++)
#define EDM_unsigned_long_from_stack()  ((unsigned long)*dsp++)
#define EDM_long_from_stack()           ((long)*dsp++)
#define EDM_DESCRIPTOR_from_stack()     ((DESCRIPTOR)(*dsp++<<3))
#define EDM_to_stack(x)                 *--dsp=(int)x

extern short EDM_answer_length;

/****************************************************************************\
  Some MESUlib definitions
\****************************************************************************/

typedef int MESU_BOOLEAN; /* 0=false, 1=true */

typedef long MESU_FIXED_16; /* with 16 bits fraction part */

struct MESU_FIXED_32
  {
  unsigned long fract_part;
  long          int_part;
  };

struct MESU_SAMPLE_STRUCT
  {
  MESU_FIXED_16 sample_x_r;
  MESU_FIXED_16 sample_y_teta;
  MESU_FIXED_16 sample_deviation;
  MESU_BOOLEAN  sample_valid;
  };

/****************************************************************************\
  EureTalk dictionary definition
\****************************************************************************/

/* END OF LIST MARKERS */
#define LAST    0
#define ILAST  -1

typedef void (*action_t)() ;        /* Action routine pointer */

struct dic1 {                       /* Loader structure */
        action_t action ;
        int start ;        } ;

struct dic3 {                       /* Loader structure 2 */
        int offset ;
        int start ;   } ;

/****************************************************************************\
  GSP Video registers
\****************************************************************************/

#define GSP_HESYNC   *(short*)0xC0000000
#define GSP_HEBLNK   *(short*)0xC0000010
#define GSP_HSBLNK   *(short*)0xC0000020
#define GSP_HTOTAL   *(short*)0xC0000030
#define GSP_VESYNC   *(short*)0xC0000040
#define GSP_VEBLNK   *(short*)0xC0000050
#define GSP_VSBLNK   *(short*)0xC0000060
#define GSP_VTOTAL   *(short*)0xC0000070
#define GSP_DPYCTL   *(short*)0xC0000080
#define GSP_DPYSTRT  *(short*)0xC0000090
#define GSP_DPYINT   *(short*)0xC00000A0
#define GSP_CONTROL  *(short*)0xC00000B0
#define GSP_HSTDATA  *(short*)0xC00000C0
#define GSP_HSTADRL  *(short*)0xC00000D0
#define GSP_HSTADRH  *(short*)0xC00000E0
#define GSP_HSTCTLL  *(short*)0xC00000F0
#define GSP_HSTCTLH  *(short*)0xC0000100
#define GSP_INTENB   *(short*)0xC0000110
#define GSP_INTPEND  *(volatile short*)0xC0000120
#define GSP_CONVSP   *(short*)0xC0000130
#define GSP_CONVDP   *(short*)0xC0000140
#define GSP_PSIZE    *(short*)0xC0000150
#define GSP_PMASK    *(short*)0xC0000160
#define GSP_PMASK32  *(long*)0xC0000160
#define GSP_DPYTAP   *(short*)0xC00001B0
#define GSP_HCOUNT   *(volatile short*)0xC00001C0
#define GSP_VCOUNT   *(volatile short*)0xC00001D0
#define GSP_DPYADR   *(volatile short*)0xC00001E0
#define GSP_REFCNT   *(volatile short*)0xC00001F0

/****************************************************************************\
  ASM definition of GSP Video registers
\****************************************************************************/

asm ("HESYNC         .set 0C0000000H");
asm ("HEBLNK         .set 0C0000010H");
asm ("HSBLNK         .set 0C0000020H");
asm ("HTOTAL         .set 0C0000030H");
asm ("VESYNC         .set 0C0000040H");
asm ("VEBLNK         .set 0C0000050H");
asm ("VSBLNK         .set 0C0000060H");
asm ("VTOTAL         .set 0C0000070H");
asm ("DPYCTL         .set 0C0000080H");
asm ("DPYSTRT        .set 0C0000090H");
asm ("DPYINT         .set 0C00000A0H");
asm ("CONTROL        .set 0C00000B0H");
asm ("HSTDATA        .set 0C00000C0H");
asm ("HSTADRL        .set 0C00000D0H");
asm ("HSTADRH        .set 0C00000E0H");
asm ("HSTCTLL        .set 0C00000F0H");
asm ("HSTCTLH        .set 0C0000100H");
asm ("INTENB         .set 0C0000110H");
asm ("INTPEND        .set 0C0000120H");
asm ("CONVSP         .set 0C0000130H");
asm ("CONVDP         .set 0C0000140H");
asm ("PSIZE          .set 0C0000150H");
asm ("PMASK          .set 0C0000160H");
asm ("DPYTAP         .set 0C00001B0H");
asm ("HCOUNT         .set 0C00001C0H");
asm ("VCOUNT         .set 0C00001D0H");
asm ("DPYADR         .set 0C00001E0H");
asm ("REFCNT         .set 0C00001F0H");

asm("CONTROL_T        .set 0020H");
asm("CONTROL_W        .set 00C0H");
asm("CONTROL_PPOP     .set 7C00H");
asm("CONTROL_PPOP_0   .set 0C00H");
asm("CONTROL_PPOP_NEG .set 1800H");
asm("CONTROL_PPOP_1   .set 3000H");

asm("SADDR   .set B0");
asm("SPTCH   .set B1");
asm("DADDR   .set B2");
asm("DPTCH   .set B3");
asm("OFFSET  .set B4");
asm("WSTART  .set B5");
asm("WEND    .set B6");
asm("DYDX    .set B7");
asm("COLOR0  .set B8");
asm("COLOR1  .set B9");
asm("COUNT   .set B10");
asm("INC1    .set B11");
asm("INC2    .set B12");
asm("PATTRN  .set B13");
asm("BTEMP   .set B14");

asm("REG_4   .set A0");
asm("ATEMP0  .set A1");
asm("REG_5   .set A2");
asm("ATEMP1  .set A3");
asm("REG_6   .set A4");
asm("ATEMP2  .set A5");
asm("REG_7   .set A6");
asm("ATEMP3  .set A7");
asm("RET_VAL .set A8");
asm("REG_0   .set A9");
asm("REG_1   .set A10");
asm("REG_2   .set A11");
asm("REG_3   .set A12");
asm("BP      .set A13");
asm("TP      .set A14");
#endif

/****************************************************************************\
  SoloCom definitions
\****************************************************************************/

#define DOWNLOAD_MIN_BAUD_RATE         600
#define DOWNLOAD_MAX_BAUD_RATE       38400L
#define DOWNLOAD_DEFAULT_BAUD_RATE    9600

#define MOUSE_LEFT_BUTTON              0x1
#define MOUSE_RIGHT_BUTTON             0x2
#define MOUSE_MIDDLE_BUTTON            0x4

#define MOUSE_TYPE_MS      1
#define MOUSE_TYPE_GENIUS  2

#define KEYCODESIZE  126
#define KEYCODEWIDTH 5

#define EURE_KEY_BREAK_CODE 0xF0

#define EURE_KEY_NORMAL    1
#define EURE_KEY_SHIFT     2
#define EURE_KEY_CNTRL     3
#define EURE_KEY_ALT       4

#define EURE_KEY_CAPLCK    5
#define EURE_KEY_STATUS    6
#define EURE_KEY_NUMLCK    7

#define EURE_KEY_NUM_LOCK  8
#define EURE_KEY_CAPS_LOCK 9

#define SCANTOKEYSIZE 133

#define DOWNLOAD_SRAM_USER_VECT 1
#define DOWNLOAD_FULL           2

/****************************************************************************\
  Definition of EDM global variables (PC environment only)
\****************************************************************************/

#ifndef _TMS340

extern unsigned short EDM_send_time_out;
extern unsigned short EDM_receive_time_out;
extern unsigned short EDM_PC_interrupt;
extern unsigned char  EDM_target;
extern unsigned short EDM_banner;
extern unsigned char  EDM_present_eurecard;
extern unsigned short EDM_connected_eurecard;
extern unsigned short EDM_n_eurecard;
extern unsigned short EDM_PC_wait_state;
extern unsigned short EDM_location_eurecard[];
extern unsigned short EDM_model_eurecard[];
extern unsigned short EDM_model_connected_eurecard;
extern unsigned short EDM_load_fonts;
extern unsigned short EDM_load_lca;
extern unsigned short EDM_load_tim;
extern unsigned short EDM_setup_primo_vga;
extern unsigned short EDM_cold_boot;
extern unsigned short EDM_default_timing;

#endif

/****************************************************************************\
  Declaration of Esope descriptors for the GSP environment
\****************************************************************************/

#ifdef _TMS340

extern DESCRIPTOR

BLK,
QAD,
QAU,
QBD,
QBU,
INFO_BLK,
LIVE_BLK,
GRAB,
GRAB_RGB,
GRAB_ZOOM_OUT,
VISU,
VISU_RGB,
VISU_R,
VISU_G,
VISU_B,
VISU_ZOOM_IN,
VISU_ZOOM_OUT,
OVL,
OVL_RGB,
OVL_R,
OVL_G,
OVL_B,
NO_OVL,
Q0AD,
Q1AD,
Q2AD,
Q3AD,
Q0AU,
Q1AU,
Q2AU,
Q3AU,
Q0BD,
Q1BD,
Q2BD,
Q3BD,
Q0BU,
Q1BU,
Q2BU,
Q3BU,
CAD,
CAU,
CBD,
CBU,
E0AD,
E1AD,
E0AU,
E1AU,
E0BD,
E1BD,
E0BU,
E1BU,
MRGB,
QRGB,
Q0RGB,
Q1RGB,
Q2RGB,
Q3RGB,
MAD,
MAU,
MBD,
MBU,
CRGB,
E0RGB,
E1RGB,
M_FRAME_BLK,
OLD_INFO_BLK,
TEMP_VISU_BLK,
NO_VISU,
JPEG_BUFFER,
BUS_BLK,

IMAGE_0,
ROI_0,
IMAGE_1,
ROI_1,
IMAGE_2,
ROI_2,
IMAGE_3,
ROI_3,

APL_LCA,
LCA,
MAP_LCA,
VFC_LCA,
PTS_LCA,
CTG_LCA,
GLC_LCA,

APL_VIN,
VIN,

APL_SYNC,
SYNC,

TIM,
APL_TIM,
APL_ACQ_TIM,
APL_VISU_TIM,

TCNIN,
TENIN,

FULL_EIA_TIM,
FULL_CCIR_TIM,
SQUARE_EIA_TIM,
SQUARE_CCIR_TIM,
HEX_EIA_TIM,
HEX_CCIR_TIM,
BDCST_EIA_TIM,
BDCST_CCIR_TIM,
CCD_EIA_TIM,
CCD_CCIR_TIM,
HIGH_EIA_TIM,
HIGH_CCIR_TIM,
CID2250_TIM,

VGA_640_480_TIM,
VGA_800_600_TIM,
VGA_1024_768_TIM,
VISU_FULL_VGA_TIM,
VISU_SQUARE_VGA_TIM,

DIO,
APL_DIO,

COM,
APL_COM,

VOUT,
APL_VOUT,

PEN,
DEL_PEN,

A_BLACK_PEN,
A_BLUE_PEN,
A_RED_PEN,
A_MAGENTA_PEN,
A_GREEN_PEN,
A_CYAN_PEN,
A_YELLOW_PEN,
A_WHITE_PEN,

G_BLACK_PEN,
G_BLUE_PEN,
G_RED_PEN,
G_MAGENTA_PEN,
G_GREEN_PEN,
G_CYAN_PEN,
G_YELLOW_PEN,
G_WHITE_PEN,

G_ORANGE_PEN,
G_OLIVE_PEN,
G_BROWN_PEN,
G_SKY_PEN,
G_PINK_PEN,
G_BRICK_PEN,
G_GREY_PEN,

LIGHT_PEN,
CURSOR_PEN,

FNT,
SIZE_8_FNT,
SIZE_16_FNT,
IBM_8_FNT,
IBM_10_FNT,
IBM_14_FNT,
IBM_16_FNT,
CURSOR_FNT,

VECT,
RAMP_VECT,
BIN_VECT,
COLOR_VECT,
INVERT_VECT,
SKELET1_VECT,
SKELET2_VECT,
NEXP_VECT,
VEXP_VECT,
NLOG_VECT,
VLOG_VECT,
SATURATION_VECT,
SQUARE_ROOT_VECT,
SQUARE_VECT,
THERMAL_VECT,
UNSIGN_VECT,
SRAM_USER_VECT,

VGA_CTRL_VECT,
PRIMO_OVL_VOUT_VECT,
PRIMO_OVL_VIN_VECT,

HIST_VECT,
PROJ_VECT,
PROFILE_VECT,
MESU_VECT,

KRNL,
SOBELX_KRNL,
SOBELY_KRNL,
PREWITTX_KRNL,
PREWITTY_KRNL,
LAPLACIAN4_KRNL,
LAPLACIAN8_KRNL,
LOWPASS1_KRNL,
LOWPASS2_KRNL,
LOWPASS3_KRNL,
LOWPASS5_KRNL,
HIGHPASS1_KRNL,
HIGHPASS2_KRNL,
WHITE_SKELET_KRNL,
BLACK_SKELET_KRNL,
EDGE_KRNL,

APL_EVA,
EVA,

CBLK,
REF_CBLK,
INSP_CBLK,

OCR,

BIC,
APL_BIC,

CONFIG,

MESU,
MESU_FIND_POINT,
MESU_FIT_LINE,
MESU_FIT_CIRCLE,

JPEG,
APL_JPEG;

#endif

/****************************************************************************\
  Declaration of Esope library functions for the PC environment
\****************************************************************************/

#ifndef _TMS340

short __far __cdecl gsp_brioboot(const char __far *);
short __far __cdecl gsp_soloboot(const char __far *);
short __far __cdecl gsp_primoboot(const char __far *);
short __far __cdecl gsp_eureboot(const char __far *,const char __far *,const char __far *);

short __far __cdecl gsp_set_connect(unsigned short);
short __far __cdecl gsp_get_connect(unsigned short __far *);

short __far __cdecl gsp_create(DESCRIPTOR __far *,DESCRIPTOR,const char __far *);
short __far __cdecl gsp_create_load(DESCRIPTOR __far *,DESCRIPTOR,const char __far *,const char __far *);
short __far __cdecl gsp_copy(DESCRIPTOR,DESCRIPTOR);
short __far __cdecl gsp_set(DESCRIPTOR,unsigned short,long);
short __far __cdecl gsp_get(DESCRIPTOR,unsigned short,long __far *);
short __far __cdecl gsp_get_event(DESCRIPTOR,unsigned short,long __far *);
long  __far __cdecl gsp_parameter(DESCRIPTOR,unsigned short);
short __far __cdecl gsp_read_data(DESCRIPTOR,unsigned short,unsigned short,unsigned short,void __far *);
short __far __cdecl gsp_write_data(DESCRIPTOR,unsigned short,unsigned short,unsigned short,void __far *);

short __far __cdecl gsp_save(DESCRIPTOR,const char __far *);
short __far __cdecl gsp_load(DESCRIPTOR,const char __far *);

short __far __cdecl gsp_clear_vect_data(DESCRIPTOR);

short __far __cdecl gsp_set_blk(DESCRIPTOR,unsigned short,unsigned short,unsigned short,unsigned short);
short __far __cdecl gsp_get_blk(DESCRIPTOR,unsigned short __far *,unsigned short __far *,unsigned short __far *,unsigned short __far *);

short __far __cdecl gsp_clear_video_mem(void);
short __far __cdecl gsp_clear_image_mem(void);
short __far __cdecl gsp_set_sync_mode(unsigned short);
short __far __cdecl gsp_locked(void);

short __far __cdecl map_12(unsigned short,DESCRIPTOR,DESCRIPTOR);
short __far __cdecl gsp_abort(void);

short __far __cdecl gsp_line(DESCRIPTOR,DESCRIPTOR,short,short,short,short);
short __far __cdecl gsp_rectangle(DESCRIPTOR,DESCRIPTOR,short,short,short,short);
short __far __cdecl gsp_rectangle_fill(DESCRIPTOR,DESCRIPTOR,short,short,short,short);
short __far __cdecl gsp_ellipse(DESCRIPTOR,DESCRIPTOR,short,short,short,short);
short __far __cdecl gsp_ellipse_fill(DESCRIPTOR,DESCRIPTOR,short,short,short,short);
short __far __cdecl gsp_conic_spline(DESCRIPTOR,DESCRIPTOR,short,short,short,short,short,short);
short __far __cdecl gsp_circle_arc(DESCRIPTOR,DESCRIPTOR,short,short,short,short,short,long);
short __far __cdecl gsp_ellipse_arc(DESCRIPTOR,DESCRIPTOR,short,short,short,short,short,long,long);
short __far __cdecl gsp_poly_line(DESCRIPTOR,DESCRIPTOR,DESCRIPTOR);
short __far __cdecl gsp_draw_contour(DESCRIPTOR,DESCRIPTOR,DESCRIPTOR);
short __far __cdecl gsp_profile(DESCRIPTOR,DESCRIPTOR,DESCRIPTOR,unsigned short,short,short,long,long,unsigned short);
short __far __cdecl gsp_fill_area(DESCRIPTOR,DESCRIPTOR,short,short);
short __far __cdecl gsp_draw_bitmap(DESCRIPTOR,unsigned short *,short,short,short,short);

short __far __cdecl gsp_write_text(DESCRIPTOR,DESCRIPTOR,DESCRIPTOR,short,short,const char __far *);
short __far __cdecl gsp_write_cursor(DESCRIPTOR,DESCRIPTOR,DESCRIPTOR,short,short,unsigned short);
short __far __cdecl gsp_move_cursor(DESCRIPTOR,DESCRIPTOR,DESCRIPTOR,short,short,unsigned short);
short __far __cdecl gsp_cursor_off(void);

short __far __cdecl gsp_op_11(unsigned short,DESCRIPTOR);
short __far __cdecl gsp_op_c11(unsigned short,short,DESCRIPTOR);
short __far __cdecl gsp_op_12(unsigned short,DESCRIPTOR,DESCRIPTOR);
short __far __cdecl gsp_op_122(unsigned short,DESCRIPTOR,DESCRIPTOR);
short __far __cdecl gsp_op_123(unsigned short,DESCRIPTOR,DESCRIPTOR,DESCRIPTOR);
short __far __cdecl gsp_op_c12(unsigned short,short,DESCRIPTOR,DESCRIPTOR);
short __far __cdecl gsp_move_blk(DESCRIPTOR,long,long);
short __far __cdecl gsp_focus(DESCRIPTOR,DESCRIPTOR,DESCRIPTOR,DESCRIPTOR,unsigned long *);

short __far __cdecl gsp_thr(DESCRIPTOR,DESCRIPTOR,unsigned short);
short __far __cdecl gsp_thr2(DESCRIPTOR,DESCRIPTOR,unsigned short,unsigned short,unsigned short,unsigned short,unsigned short);
short __far __cdecl gsp_lut(DESCRIPTOR,DESCRIPTOR,DESCRIPTOR);
short __far __cdecl gsp_comb_1c2(short,DESCRIPTOR,short,DESCRIPTOR);
short __far __cdecl gsp_comb_123(short,DESCRIPTOR,short,DESCRIPTOR,DESCRIPTOR);
short __far __cdecl gsp_comb_ovl(DESCRIPTOR,DESCRIPTOR,DESCRIPTOR);
short __far __cdecl gsp_normalize(DESCRIPTOR,DESCRIPTOR,long,long,long,long);
short __far __cdecl gsp_warp(DESCRIPTOR,DESCRIPTOR,DESCRIPTOR);

short __far __cdecl vfc_11(unsigned short,DESCRIPTOR);
short __far __cdecl vfc_12(unsigned short,DESCRIPTOR,DESCRIPTOR);
short __far __cdecl vfc_123(unsigned short,DESCRIPTOR,DESCRIPTOR,DESCRIPTOR);
short __far __cdecl vfc_1c1(unsigned short,DESCRIPTOR,short);
short __far __cdecl vfc_1c2(unsigned short,DESCRIPTOR,short,DESCRIPTOR);
short __far __cdecl vfc_grab_average(DESCRIPTOR,DESCRIPTOR,DESCRIPTOR,short,short);
short __far __cdecl vfc_thr(DESCRIPTOR,DESCRIPTOR,unsigned short);

short __far __cdecl rtc_op_1c2(unsigned short,DESCRIPTOR,long,DESCRIPTOR);
short __far __cdecl rtc_thr(DESCRIPTOR,DESCRIPTOR,unsigned short);
short __far __cdecl rtc_thr2(DESCRIPTOR,DESCRIPTOR,unsigned short,unsigned short,unsigned short,unsigned short,unsigned short);
short __far __cdecl rtc_lut(DESCRIPTOR,DESCRIPTOR,DESCRIPTOR);
short __far __cdecl rtc_expand(DESCRIPTOR,DESCRIPTOR);
short __far __cdecl rtc_normalize(DESCRIPTOR,DESCRIPTOR,long,long,long,long);
short __far __cdecl rtc_move(DESCRIPTOR,DESCRIPTOR);

short __far __cdecl gsp_convol(DESCRIPTOR,DESCRIPTOR,DESCRIPTOR);
short __far __cdecl gsp_lowpass1(DESCRIPTOR,DESCRIPTOR);
short __far __cdecl gsp_lowpass2(DESCRIPTOR,DESCRIPTOR);
short __far __cdecl gsp_lowpass3(DESCRIPTOR,DESCRIPTOR);
short __far __cdecl gsp_highpass1(DESCRIPTOR,DESCRIPTOR);
short __far __cdecl gsp_highpass2(DESCRIPTOR,DESCRIPTOR);
short __far __cdecl gsp_sobelx(DESCRIPTOR,DESCRIPTOR);
short __far __cdecl gsp_sobely(DESCRIPTOR,DESCRIPTOR);
short __far __cdecl gsp_sobelxy(DESCRIPTOR,DESCRIPTOR);
short __far __cdecl gsp_prewitx(DESCRIPTOR,DESCRIPTOR);
short __far __cdecl gsp_prewity(DESCRIPTOR,DESCRIPTOR);
short __far __cdecl gsp_prewitxy(DESCRIPTOR,DESCRIPTOR);

short __far __cdecl rtc_convol(DESCRIPTOR,DESCRIPTOR,DESCRIPTOR);
short __far __cdecl rtc_convol2(DESCRIPTOR,DESCRIPTOR,DESCRIPTOR,DESCRIPTOR);

short __far __cdecl gsp_dilate(DESCRIPTOR,DESCRIPTOR);
short __far __cdecl gsp_erode(DESCRIPTOR,DESCRIPTOR);
short __far __cdecl gsp_open(DESCRIPTOR,DESCRIPTOR);
short __far __cdecl gsp_close(DESCRIPTOR,DESCRIPTOR);
short __far __cdecl gsp_ndilate(DESCRIPTOR,DESCRIPTOR,unsigned short);
short __far __cdecl gsp_nerode(DESCRIPTOR,DESCRIPTOR,unsigned short);
short __far __cdecl gsp_nopen(DESCRIPTOR,DESCRIPTOR,unsigned short);
short __far __cdecl gsp_nclose(DESCRIPTOR,DESCRIPTOR,unsigned short);
short __far __cdecl gsp_thin(DESCRIPTOR,DESCRIPTOR,DESCRIPTOR,unsigned short,unsigned short __far *);
short __far __cdecl gsp_thick(DESCRIPTOR,DESCRIPTOR,DESCRIPTOR,unsigned short,unsigned short __far *);
short __far __cdecl gsp_distance(DESCRIPTOR,DESCRIPTOR);

short __far __cdecl rtc_erode(DESCRIPTOR,DESCRIPTOR);
short __far __cdecl rtc_dilate(DESCRIPTOR,DESCRIPTOR);
short __far __cdecl rtc_open(DESCRIPTOR,DESCRIPTOR);
short __far __cdecl rtc_close(DESCRIPTOR,DESCRIPTOR);
short __far __cdecl rtc_nerode(DESCRIPTOR,DESCRIPTOR,unsigned short);
short __far __cdecl rtc_ndilate(DESCRIPTOR,DESCRIPTOR,unsigned short);
short __far __cdecl rtc_thin(DESCRIPTOR,DESCRIPTOR,DESCRIPTOR,unsigned short,unsigned short __far *);
short __far __cdecl rtc_thick(DESCRIPTOR,DESCRIPTOR,DESCRIPTOR,unsigned short,unsigned short __far *);
short __far __cdecl rtc_pattern_code(DESCRIPTOR,DESCRIPTOR,DESCRIPTOR,DESCRIPTOR,unsigned short __far *);

short __far __cdecl gsp_grey_erode(DESCRIPTOR,DESCRIPTOR,unsigned short,unsigned short);
short __far __cdecl gsp_grey_dilate(DESCRIPTOR,DESCRIPTOR,unsigned short,unsigned short);
short __far __cdecl gsp_grey_median(DESCRIPTOR,DESCRIPTOR,unsigned short);
short __far __cdecl glc_12(unsigned short,DESCRIPTOR,DESCRIPTOR);
short __far __cdecl mmc_12(unsigned short,DESCRIPTOR,DESCRIPTOR);

short __far __cdecl gsp_hex_bin_erode(DESCRIPTOR,DESCRIPTOR,unsigned short,unsigned short);
short __far __cdecl gsp_hex_bin_dilate(DESCRIPTOR,DESCRIPTOR,unsigned short,unsigned short);
short __far __cdecl gsp_hex_bin_borderkill(DESCRIPTOR,DESCRIPTOR,unsigned short);
short __far __cdecl gsp_hex_bin_holefill(DESCRIPTOR,DESCRIPTOR,unsigned short);
short __far __cdecl gsp_hex_bin_reconstruct(DESCRIPTOR,DESCRIPTOR);
short __far __cdecl gsp_hex_erode(DESCRIPTOR,DESCRIPTOR,unsigned short,unsigned short);
short __far __cdecl gsp_hex_dilate(DESCRIPTOR,DESCRIPTOR,unsigned short,unsigned short);
short __far __cdecl gsp_hex_open(DESCRIPTOR,DESCRIPTOR,unsigned short,unsigned short);
short __far __cdecl gsp_hex_close(DESCRIPTOR,DESCRIPTOR,unsigned short,unsigned short);
short __far __cdecl gsp_hex_lin_erode(DESCRIPTOR,DESCRIPTOR,unsigned short,unsigned short,unsigned short);
short __far __cdecl gsp_hex_lin_dilate(DESCRIPTOR,DESCRIPTOR,unsigned short,unsigned short,unsigned short);
short __far __cdecl gsp_hex_lin_open(DESCRIPTOR,DESCRIPTOR,unsigned short,unsigned short,unsigned short);
short __far __cdecl gsp_hex_lin_close(DESCRIPTOR,DESCRIPTOR,unsigned short,unsigned short,unsigned short);
short __far __cdecl gsp_hex_distance(DESCRIPTOR,DESCRIPTOR,unsigned short);
short __far __cdecl gsp_hex_geoprop(DESCRIPTOR,DESCRIPTOR,unsigned short,unsigned short,unsigned short,unsigned short);
short __far __cdecl gsp_hex_reconstruct(DESCRIPTOR,DESCRIPTOR);

short __far __cdecl gsp_set_grid(unsigned short);
short __far __cdecl gsp_set_connectivity(unsigned short);
short __far __cdecl rtc2_set_functionality(unsigned short);

short __far __cdecl gsp_read_timer(unsigned long __far *);
short __far __cdecl gsp_write_timer(unsigned long);
short __far __cdecl gsp_enable_timer(void);
short __far __cdecl gsp_disable_timer(void);
short __far __cdecl gsp_read_fct_time(unsigned long __far *);
short __far __cdecl print_gsp_time(const char *);

short __far __cdecl gsp_pixel_min(DESCRIPTOR,unsigned short __far *);
short __far __cdecl gsp_pixel_max(DESCRIPTOR,unsigned short __far *);
short __far __cdecl gsp_pixel_average(DESCRIPTOR,long __far *);
short __far __cdecl gsp_pixel_std_deviation(DESCRIPTOR,long __far *);
short __far __cdecl gsp_pixel_stat(DESCRIPTOR,unsigned short __far *,unsigned short __far *,unsigned short __far *);
short __far __cdecl rtc_stat(DESCRIPTOR,short __far *,short __far *);

short __far __cdecl gsp_area(DESCRIPTOR,unsigned short,unsigned long __far *);
short __far __cdecl gsp_area2(DESCRIPTOR,unsigned short,unsigned short,unsigned long __far *);
short __far __cdecl gsp_count(DESCRIPTOR,unsigned short,unsigned short,
                    unsigned long __far *,unsigned long __far *,unsigned long __far *);

short __far __cdecl rtc_area(DESCRIPTOR,unsigned short,unsigned long __far *);
short __far __cdecl rtc_area2(DESCRIPTOR,unsigned short,unsigned short,unsigned long __far *);
short __far __cdecl rtc_count(DESCRIPTOR,unsigned short,unsigned short,
                    unsigned long __far *,unsigned long __far *,unsigned long __far *);

short __far __cdecl gsp_gravity_center(DESCRIPTOR,unsigned short __far *,unsigned short __far *);

short __far __cdecl gsp_hist(DESCRIPTOR,DESCRIPTOR);
short __far __cdecl gsp_proj_row(DESCRIPTOR,DESCRIPTOR);
short __far __cdecl gsp_proj_col(DESCRIPTOR,DESCRIPTOR);
short __far __cdecl rtc_bin_proj_row(DESCRIPTOR,unsigned short,DESCRIPTOR);
short __far __cdecl hst_hist(DESCRIPTOR,DESCRIPTOR);
short __far __cdecl hst_dyn_hist(DESCRIPTOR,DESCRIPTOR,DESCRIPTOR);
short __far __cdecl gsp_hist_analyse(DESCRIPTOR,unsigned short,unsigned short,unsigned long,long __far *);

short __far __cdecl hpc_hist(DESCRIPTOR,DESCRIPTOR);
short __far __cdecl hpc_dyn_hist(DESCRIPTOR,DESCRIPTOR,DESCRIPTOR);
short __far __cdecl hpc_proj_row(DESCRIPTOR,DESCRIPTOR);
short __far __cdecl hpc_proj_col(DESCRIPTOR,DESCRIPTOR);

short __far __cdecl gsp_seg_to_vect(DESCRIPTOR,short,short,short,short,DESCRIPTOR);
short __far __cdecl gsp_vect_to_seg(DESCRIPTOR,DESCRIPTOR,short,short,short,short);
short __far __cdecl gsp_trans_vect(DESCRIPTOR,DESCRIPTOR,long);
short __far __cdecl gsp_extrem_vect(DESCRIPTOR,DESCRIPTOR,long,long);
short __far __cdecl gsp_convol_vect(DESCRIPTOR,DESCRIPTOR,DESCRIPTOR);
short __far __cdecl gsp_vect_transfer(DESCRIPTOR,DESCRIPTOR,unsigned short);
short __far __cdecl gsp_vect_data_shift(DESCRIPTOR,short);
short __far __cdecl gsp_vect_1c2(unsigned short,DESCRIPTOR,long,DESCRIPTOR);

short __far __cdecl gsp_contour(DESCRIPTOR,short,short,short,unsigned short,DESCRIPTOR);
short __far __cdecl gsp_trans_scan(DESCRIPTOR,short,short,short,short,short,unsigned short,unsigned short,DESCRIPTOR);

short __far __cdecl gsp_code(DESCRIPTOR,unsigned short,unsigned short,DESCRIPTOR);
short __far __cdecl gsp_process(DESCRIPTOR,unsigned short,unsigned short,DESCRIPTOR);
short __far __cdecl gsp_dyn_code(DESCRIPTOR,DESCRIPTOR,DESCRIPTOR,DESCRIPTOR);
short __far __cdecl gsp_dyn_process(DESCRIPTOR,DESCRIPTOR,DESCRIPTOR,DESCRIPTOR);

short __far __cdecl bic_code(DESCRIPTOR,unsigned short,unsigned short,DESCRIPTOR);
short __far __cdecl bic_process(DESCRIPTOR,unsigned short,unsigned short,DESCRIPTOR);
short __far __cdecl bic_dyn_code(DESCRIPTOR,DESCRIPTOR,DESCRIPTOR,DESCRIPTOR);
short __far __cdecl bic_dyn_process(DESCRIPTOR,DESCRIPTOR,DESCRIPTOR,DESCRIPTOR);

short __far __cdecl gsp_reconstruct(DESCRIPTOR);
short __far __cdecl gsp_analyse(DESCRIPTOR,unsigned long);
short __far __cdecl gsp_feature_select(DESCRIPTOR,unsigned long,unsigned long,unsigned long,unsigned short);
short __far __cdecl gsp_location_select(DESCRIPTOR,DESCRIPTOR,unsigned short);
short __far __cdecl gsp_sort_object(DESCRIPTOR,unsigned long,unsigned short);
short __far __cdecl gsp_draw_object(DESCRIPTOR,DESCRIPTOR,unsigned short);
short __far __cdecl gsp_draw_object_using_pen(DESCRIPTOR,DESCRIPTOR,DESCRIPTOR,unsigned short,unsigned short);
short __far __cdecl gsp_read_object(DESCRIPTOR,unsigned long,unsigned short,unsigned short,unsigned long __far *);
short __far __cdecl gsp_read_all_object(DESCRIPTOR,unsigned short,unsigned short,unsigned long __far *);
short __far __cdecl gsp_num_selected_segments(DESCRIPTOR,unsigned short,unsigned short,unsigned short __far *);
short __far __cdecl gsp_read_segment(DESCRIPTOR,unsigned short,unsigned short,unsigned short __far *);
short __far __cdecl gsp_coord2object(DESCRIPTOR,short,short,long __far *);
short __far __cdecl gsp_feature_stat(DESCRIPTOR,unsigned long,long __far *,long __far *,long __far *);
short __far __cdecl gsp_create_mask(DESCRIPTOR,DESCRIPTOR,short);
short __far __cdecl gsp_associate_objects(DESCRIPTOR,DESCRIPTOR,DESCRIPTOR,DESCRIPTOR);
short __far __cdecl gsp_detect_defects(DESCRIPTOR,DESCRIPTOR,DESCRIPTOR,long,short,long,long,short,short);

short __far __cdecl ocr_show_character(DESCRIPTOR,DESCRIPTOR,DESCRIPTOR,short);
short __far __cdecl ocr_get_text(DESCRIPTOR,char *);
short __far __cdecl ocr_get_scores(DESCRIPTOR,long *,long *);
short __far __cdecl ocr_get_positions(DESCRIPTOR,long *,long *,long *,long *);
short __far __cdecl ocr_set_class(DESCRIPTOR,unsigned short,short,short);
short __far __cdecl ocr_add_to_database(DESCRIPTOR,DESCRIPTOR,short,unsigned char,unsigned short);
short __far __cdecl ocr_clear_database(DESCRIPTOR);
short __far __cdecl ocr_read_image(DESCRIPTOR,DESCRIPTOR,DESCRIPTOR,DESCRIPTOR,short,long,char *);
short __far __cdecl ocr_analyse_image(DESCRIPTOR,DESCRIPTOR,DESCRIPTOR,short,short);
short __far __cdecl ocr_read_cblk(DESCRIPTOR,DESCRIPTOR,DESCRIPTOR,char *);
short __far __cdecl ocr_analyse_cblk(DESCRIPTOR,DESCRIPTOR);

short __far __cdecl gsp_read_blk_pixel(DESCRIPTOR,unsigned short,unsigned short,unsigned short __far *);
short __far __cdecl gsp_write_blk_pixel(DESCRIPTOR,unsigned short,unsigned short,unsigned short);
short __far __cdecl gsp_read_blk_row(DESCRIPTOR,unsigned short,unsigned short __far *);
short __far __cdecl gsp_write_blk_row(DESCRIPTOR,unsigned short,unsigned short __far *);
short __far __cdecl gsp_read_blk_col(DESCRIPTOR,unsigned short,unsigned short __far *);
short __far __cdecl gsp_write_blk_col(DESCRIPTOR,unsigned short,unsigned short __far *);
short __far __cdecl gsp_read_blk(DESCRIPTOR,void *);
short __far __cdecl gsp_read_blk_8(DESCRIPTOR,void *);
short __far __cdecl gsp_read_blk_16(DESCRIPTOR,void *);
short __far __cdecl gsp_save_blk(DESCRIPTOR,const char __far *,unsigned short);
short __far __cdecl gsp_load_blk(DESCRIPTOR,const char __far *);

short __far __cdecl gsp_initialize_blk_transfer(DESCRIPTOR);
short __far __cdecl gsp_end_blk_transfer(void);
short __far __cdecl gsp_read_row(unsigned short,unsigned short __far *);
short __far __cdecl gsp_write_row(unsigned short,unsigned short __far *);
short __far __cdecl gsp_read_col(unsigned short,unsigned short __far *);
short __far __cdecl gsp_write_col(unsigned short,unsigned short __far *);

short __far __cdecl gsp_execute_C_command(unsigned long __far *);
short __far __cdecl gsp_debug_C_command(unsigned long __far *);
short __far __cdecl gsp_execute_C_function(unsigned long,short,short,...);
short __far __cdecl gsp_debug_C_function(unsigned long,short,short,...);
short __far __cdecl gsp_send_msg(unsigned short,const void __far *,unsigned short);
short __far __cdecl gsp_receive_msg(unsigned short,unsigned short __far *,void __far *,unsigned short);
short __far __cdecl gsp_create_ET_function(FUNCTION __far *,const char __far *,const char __far *);
short __far __cdecl gsp_execute_ET_function(FUNCTION);
short __far __cdecl gsp_push(unsigned long);
short __far __cdecl gsp_push_n(unsigned short,unsigned long __far *);
short __far __cdecl gsp_pop(unsigned long __far *);
short __far __cdecl gsp_pop_n(unsigned short,unsigned long __far *);
short __far __cdecl gsp_transmit(const char __far *);
short __far __cdecl gsp_receive(char __far *);
short __far __cdecl gsp_transmit_ET_file(const char __far *);
short __far __cdecl gsp_submit_ET(const char __far *);
short __far __cdecl gsp_send_break(void);

short __far __cdecl eva_lut(DESCRIPTOR);
short __far __cdecl eva_lut_fill(unsigned short);
short __far __cdecl eva_lut_transform(long,unsigned short);
short __far __cdecl eva_profile(DESCRIPTOR);
short __far __cdecl eva_profile_fill(unsigned short);
short __far __cdecl eva_set_connect(unsigned short);

short __far __cdecl mesu_measure_point(DESCRIPTOR,DESCRIPTOR,short *,long *);
short __far __cdecl mesu_measure_point_xy(DESCRIPTOR,DESCRIPTOR,short *,long *,long *);
short __far __cdecl mesu_measure_projection(DESCRIPTOR,DESCRIPTOR,short,short *,long *);
short __far __cdecl mesu_measure_projection_xy(DESCRIPTOR,DESCRIPTOR,short,short *,long *,long *);
short __far __cdecl mesu_evaluate_point_count(DESCRIPTOR);
short __far __cdecl mesu_select_point(DESCRIPTOR,long);
short __far __cdecl mesu_measure_outline(DESCRIPTOR,DESCRIPTOR,DESCRIPTOR);

short __far __cdecl jpeg_copy(DESCRIPTOR,DESCRIPTOR);
short __far __cdecl jpeg_compress(DESCRIPTOR,DESCRIPTOR);
short __far __cdecl jpeg_decompress(DESCRIPTOR,DESCRIPTOR);

short __far __cdecl com_send_char(short channel,char data);
short __far __cdecl com_receive_char(short channel,char *data);
short __far __cdecl com_send_vect(short channel,DESCRIPTOR vect);
short __far __cdecl com_receive_vect(short channel,DESCRIPTOR vect);
short __far __cdecl keyboard_init(short,short,short *);
short __far __cdecl keyboard_getch(short *);
short __far __cdecl mouse_init(short,short,short,short,short,short,short,short,short);
short __far __cdecl mouse_reset(short,short,short,short,short,short,short);
short __far __cdecl mouse_get_state(short *,short *,short *);
short __far __cdecl mouse_get_click(short *,short *,short *,short *,short *,short *);

short __far __cdecl gsp_get_desc_name(int,DESCRIPTOR,int,unsigned long,DESCRIPTOR *,char *);
short __far __cdecl gsp_hide_desc(DESCRIPTOR,unsigned short);
short __far __cdecl gsp_wait_event(unsigned short);
short __far __cdecl gsp_set_expert_mode(unsigned short);

short __far __cdecl gsp_setup_safe_memory(short,short,const char *,unsigned long);

unsigned char  __far __cdecl get_EDM_present_eurecard(void);
unsigned short __far __cdecl get_EDM_connected_eurecard(void);
unsigned short __far __cdecl get_EDM_n_eurecard(void);
unsigned short __far __cdecl get_EDM_location_eurecard(short);
unsigned short __far __cdecl get_EDM_model_eurecard(short);
unsigned short __far __cdecl get_EDM_model_connected_eurecard(void);
unsigned short __far __cdecl get_EDM_cold_boot(void);

short __far __cdecl set_EDM_connected_eurecard(unsigned short);
short __far __cdecl set_EDM_send_time_out(unsigned short);
short __far __cdecl set_EDM_receive_time_out(unsigned short);
short __far __cdecl set_EDM_PC_interrupt(unsigned short);
short __far __cdecl set_EDM_target(unsigned char);
short __far __cdecl set_EDM_banner(unsigned short);
short __far __cdecl set_EDM_PC_wait_state(unsigned short);
short __far __cdecl set_EDM_load_fonts(unsigned short);
short __far __cdecl set_EDM_load_lca(unsigned short);
short __far __cdecl set_EDM_setup_primo_vga(unsigned short);
short __far __cdecl set_EDM_default_timing(unsigned short);

unsigned short __far __cdecl get_gsp_control_register(void);
void           __far __cdecl set_gsp_control_register(unsigned short);
unsigned long  __far __cdecl get_gsp_address(void);
void           __far __cdecl set_gsp_address(unsigned long);
unsigned short __far __cdecl get_gsp_data(void);
void           __far __cdecl set_gsp_data(unsigned short);
void           __far __cdecl gsp2pc(unsigned short,unsigned long,void __far *);
void           __far __cdecl pc2gsp(unsigned short,const void __far *,unsigned long);

short __far __cdecl gsp_load_primo_vga(void);
short __far __cdecl gsp_save_primo_vga(void);
short __far __cdecl gsp_setup_primo_lut(unsigned short);

short __far __cdecl gsp_free_memory(unsigned long);
short __far __cdecl gsp_alloc_memory(unsigned long,unsigned long *);
short __far __cdecl Eure_RAM_Check(short);
short __far __cdecl IsCrisplantSolo(void);


#endif

/****************************************************************************\
  Declaration of Esope library functions for the GSP environment
\****************************************************************************/

#ifdef _TMS340

void gsp_soft_reset(void);

short gsp_create(DESCRIPTOR *,DESCRIPTOR,char *);
short gsp_create_load(DESCRIPTOR *,DESCRIPTOR,char *,char *);
short gsp_copy(DESCRIPTOR,DESCRIPTOR);
short gsp_set(DESCRIPTOR,unsigned short,long);
short gsp_get(DESCRIPTOR,unsigned short,long *);
short gsp_get_event(DESCRIPTOR,unsigned short,long *);
long  gsp_parameter(DESCRIPTOR,unsigned short);
short gsp_read_data(DESCRIPTOR,unsigned short,unsigned short,unsigned short,void *);
short gsp_write_data(DESCRIPTOR,unsigned short,unsigned short,unsigned short,void *);

short gsp_clear_vect_data(DESCRIPTOR);

short gsp_set_blk(DESCRIPTOR,unsigned short,unsigned short,unsigned short,unsigned short);
short gsp_get_blk(DESCRIPTOR,unsigned short *,unsigned short *,unsigned short *,unsigned short *);

short gsp_clear_video_mem(void);
short gsp_clear_image_mem(void);
short gsp_set_sync_mode(unsigned short);
short gsp_locked(void);

short map_12(unsigned short,DESCRIPTOR,DESCRIPTOR);

short gsp_line(DESCRIPTOR,DESCRIPTOR,short,short,short,short);
short gsp_rectangle(DESCRIPTOR,DESCRIPTOR,short,short,short,short);
short gsp_rectangle_fill(DESCRIPTOR,DESCRIPTOR,short,short,short,short);
short gsp_ellipse(DESCRIPTOR,DESCRIPTOR,short,short,short,short);
short gsp_ellipse_fill(DESCRIPTOR,DESCRIPTOR,short,short,short,short);
short gsp_conic_spline(DESCRIPTOR,DESCRIPTOR,short,short,short,short,short,short);
short gsp_circle_arc(DESCRIPTOR,DESCRIPTOR,short,short,short,short,short,long);
short gsp_ellipse_arc(DESCRIPTOR,DESCRIPTOR,short,short,short,short,short,long,long);
short gsp_poly_line(DESCRIPTOR,DESCRIPTOR,DESCRIPTOR);
short gsp_draw_contour(DESCRIPTOR,DESCRIPTOR,DESCRIPTOR);
short gsp_profile(DESCRIPTOR,DESCRIPTOR,DESCRIPTOR,unsigned short,short,short,long,long,unsigned short);
short gsp_fill_area(DESCRIPTOR,DESCRIPTOR,short,short);
short gsp_draw_bitmap(DESCRIPTOR,unsigned short *,short,short,short,short);

short gsp_write_text(DESCRIPTOR,DESCRIPTOR,DESCRIPTOR,short,short,char *);
short gsp_write_cursor(DESCRIPTOR,DESCRIPTOR,DESCRIPTOR,short,short,unsigned short);
short gsp_move_cursor(DESCRIPTOR,DESCRIPTOR,DESCRIPTOR,short,short,unsigned short);
short gsp_cursor_off(void);

short gsp_op_11(unsigned short,DESCRIPTOR);
short gsp_op_c11(unsigned short,short,DESCRIPTOR);
short gsp_op_12(unsigned short,DESCRIPTOR,DESCRIPTOR);
short gsp_op_122(unsigned short,DESCRIPTOR,DESCRIPTOR);
short gsp_op_123(unsigned short,DESCRIPTOR,DESCRIPTOR,DESCRIPTOR);
short gsp_op_c12(unsigned short,short,DESCRIPTOR,DESCRIPTOR);
short gsp_move_blk(DESCRIPTOR,long,long);
short gsp_focus(DESCRIPTOR,DESCRIPTOR,DESCRIPTOR,DESCRIPTOR,unsigned long *);

short gsp_thr(DESCRIPTOR,DESCRIPTOR,unsigned short);
short gsp_thr2(DESCRIPTOR,DESCRIPTOR,unsigned short,unsigned short,unsigned short,unsigned short,unsigned short);
short gsp_lut(DESCRIPTOR,DESCRIPTOR,DESCRIPTOR);
short gsp_comb_1c2(short,DESCRIPTOR,short,DESCRIPTOR);
short gsp_comb_123(short,DESCRIPTOR,short,DESCRIPTOR,DESCRIPTOR);
short gsp_comb_ovl(DESCRIPTOR,DESCRIPTOR,DESCRIPTOR);
short gsp_normalize(DESCRIPTOR,DESCRIPTOR,long,long,long,long);
short gsp_warp(DESCRIPTOR,DESCRIPTOR,DESCRIPTOR);

short vfc_11(unsigned short,DESCRIPTOR);
short vfc_12(unsigned short,DESCRIPTOR,DESCRIPTOR);
short vfc_123(unsigned short,DESCRIPTOR,DESCRIPTOR,DESCRIPTOR);
short vfc_1c1(unsigned short,DESCRIPTOR,short);
short vfc_1c2(unsigned short,DESCRIPTOR,short,DESCRIPTOR);
short vfc_grab_average(DESCRIPTOR,DESCRIPTOR,DESCRIPTOR,short,short);
short vfc_thr(DESCRIPTOR,DESCRIPTOR,unsigned short);

short rtc_op_1c2(unsigned short,DESCRIPTOR,long,DESCRIPTOR);
short rtc_thr(DESCRIPTOR,DESCRIPTOR,unsigned short);
short rtc_thr2(DESCRIPTOR,DESCRIPTOR,unsigned short,unsigned short,unsigned short,unsigned short,unsigned short);
short rtc_lut(DESCRIPTOR,DESCRIPTOR,DESCRIPTOR);
short rtc_expand(DESCRIPTOR,DESCRIPTOR);
short rtc_normalize(DESCRIPTOR,DESCRIPTOR,long,long,long,long);
short rtc_move(DESCRIPTOR,DESCRIPTOR);

short gsp_convol(DESCRIPTOR,DESCRIPTOR,DESCRIPTOR);
short gsp_lowpass1(DESCRIPTOR,DESCRIPTOR);
short gsp_lowpass2(DESCRIPTOR,DESCRIPTOR);
short gsp_lowpass3(DESCRIPTOR,DESCRIPTOR);
short gsp_highpass1(DESCRIPTOR,DESCRIPTOR);
short gsp_highpass2(DESCRIPTOR,DESCRIPTOR);
short gsp_sobelx(DESCRIPTOR,DESCRIPTOR);
short gsp_sobely(DESCRIPTOR,DESCRIPTOR);
short gsp_sobelxy(DESCRIPTOR,DESCRIPTOR);
short gsp_prewitx(DESCRIPTOR,DESCRIPTOR);
short gsp_prewity(DESCRIPTOR,DESCRIPTOR);
short gsp_prewitxy(DESCRIPTOR,DESCRIPTOR);

short rtc_convol(DESCRIPTOR,DESCRIPTOR,DESCRIPTOR);
short rtc_convol2(DESCRIPTOR,DESCRIPTOR,DESCRIPTOR,DESCRIPTOR);

short gsp_dilate(DESCRIPTOR,DESCRIPTOR);
short gsp_erode(DESCRIPTOR,DESCRIPTOR);
short gsp_open(DESCRIPTOR,DESCRIPTOR);
short gsp_close(DESCRIPTOR,DESCRIPTOR);
short gsp_ndilate(DESCRIPTOR,DESCRIPTOR,unsigned short);
short gsp_nerode(DESCRIPTOR,DESCRIPTOR,unsigned short);
short gsp_nopen(DESCRIPTOR,DESCRIPTOR,unsigned short);
short gsp_nclose(DESCRIPTOR,DESCRIPTOR,unsigned short);
short gsp_thin(DESCRIPTOR,DESCRIPTOR,DESCRIPTOR,unsigned short,unsigned short *);
short gsp_thick(DESCRIPTOR,DESCRIPTOR,DESCRIPTOR,unsigned short,unsigned short *);
short gsp_distance(DESCRIPTOR,DESCRIPTOR);

short rtc_erode(DESCRIPTOR,DESCRIPTOR);
short rtc_dilate(DESCRIPTOR,DESCRIPTOR);
short rtc_open(DESCRIPTOR,DESCRIPTOR);
short rtc_close(DESCRIPTOR,DESCRIPTOR);
short rtc_nerode(DESCRIPTOR,DESCRIPTOR,unsigned short);
short rtc_ndilate(DESCRIPTOR,DESCRIPTOR,unsigned short);
short rtc_thin(DESCRIPTOR,DESCRIPTOR,DESCRIPTOR,unsigned short,unsigned short *);
short rtc_thick(DESCRIPTOR,DESCRIPTOR,DESCRIPTOR,unsigned short,unsigned short *);
short rtc_pattern_code(DESCRIPTOR,DESCRIPTOR,DESCRIPTOR,DESCRIPTOR,unsigned short *);

short gsp_grey_erode(DESCRIPTOR,DESCRIPTOR,unsigned short,unsigned short);
short gsp_grey_dilate(DESCRIPTOR,DESCRIPTOR,unsigned short,unsigned short);
short gsp_grey_median(DESCRIPTOR,DESCRIPTOR,unsigned short);
short glc_12(unsigned short,DESCRIPTOR,DESCRIPTOR);
short mmc_12(unsigned short,DESCRIPTOR,DESCRIPTOR);

short gsp_hex_bin_erode(DESCRIPTOR,DESCRIPTOR,unsigned short,unsigned short);
short gsp_hex_bin_dilate(DESCRIPTOR,DESCRIPTOR,unsigned short,unsigned short);
short gsp_hex_bin_borderkill(DESCRIPTOR,DESCRIPTOR,unsigned short);
short gsp_hex_bin_holefill(DESCRIPTOR,DESCRIPTOR,unsigned short);
short gsp_hex_bin_reconstruct(DESCRIPTOR,DESCRIPTOR);
short gsp_hex_erode(DESCRIPTOR,DESCRIPTOR,unsigned short,unsigned short);
short gsp_hex_dilate(DESCRIPTOR,DESCRIPTOR,unsigned short,unsigned short);
short gsp_hex_open(DESCRIPTOR,DESCRIPTOR,unsigned short,unsigned short);
short gsp_hex_close(DESCRIPTOR,DESCRIPTOR,unsigned short,unsigned short);
short gsp_hex_lin_erode(DESCRIPTOR,DESCRIPTOR,unsigned short,unsigned short,unsigned short);
short gsp_hex_lin_dilate(DESCRIPTOR,DESCRIPTOR,unsigned short,unsigned short,unsigned short);
short gsp_hex_lin_open(DESCRIPTOR,DESCRIPTOR,unsigned short,unsigned short,unsigned short);
short gsp_hex_lin_close(DESCRIPTOR,DESCRIPTOR,unsigned short,unsigned short,unsigned short);
short gsp_hex_distance(DESCRIPTOR,DESCRIPTOR,unsigned short);
short gsp_hex_geoprop(DESCRIPTOR,DESCRIPTOR,unsigned short,unsigned short,unsigned short,unsigned short);
short gsp_hex_reconstruct(DESCRIPTOR,DESCRIPTOR);

short gsp_set_grid(unsigned short);
short gsp_set_connectivity(unsigned short);
short rtc2_set_functionality(unsigned short);

short gsp_read_timer(unsigned long *);
short gsp_write_timer(unsigned long);
short gsp_enable_timer(void);
short gsp_disable_timer(void);

short gsp_pixel_min(DESCRIPTOR,unsigned short *);
short gsp_pixel_max(DESCRIPTOR,unsigned short *);
short gsp_pixel_average(DESCRIPTOR,long *);
short gsp_pixel_std_deviation(DESCRIPTOR,long *);
short gsp_pixel_stat(DESCRIPTOR,unsigned short *,unsigned short *,unsigned short *);
short rtc_stat(DESCRIPTOR,short *,short *);

short gsp_area(DESCRIPTOR,unsigned short,unsigned long *);
short gsp_area2(DESCRIPTOR,unsigned short,unsigned short,unsigned long *);
short gsp_count(DESCRIPTOR,unsigned short,unsigned short,
                    unsigned long *,unsigned long *,unsigned long *);

short rtc_area(DESCRIPTOR,unsigned short,unsigned long *);
short rtc_area2(DESCRIPTOR,unsigned short,unsigned short,unsigned long *);
short rtc_count(DESCRIPTOR,unsigned short,unsigned short,
                    unsigned long *,unsigned long *,unsigned long *);

short gsp_gravity_center(DESCRIPTOR,unsigned short *,unsigned short *);

short gsp_hist(DESCRIPTOR,DESCRIPTOR);
short gsp_proj_row(DESCRIPTOR,DESCRIPTOR);
short gsp_proj_col(DESCRIPTOR,DESCRIPTOR);
short rtc_bin_proj_row(DESCRIPTOR,unsigned short,DESCRIPTOR);
short hst_hist(DESCRIPTOR,DESCRIPTOR);
short hst_dyn_hist(DESCRIPTOR,DESCRIPTOR,DESCRIPTOR);
short gsp_hist_analyse(DESCRIPTOR,unsigned short,unsigned short,unsigned long,long *);

short hpc_hist(DESCRIPTOR,DESCRIPTOR);
short hpc_dyn_hist(DESCRIPTOR,DESCRIPTOR,DESCRIPTOR);
short hpc_proj_row(DESCRIPTOR,DESCRIPTOR);
short hpc_proj_col(DESCRIPTOR,DESCRIPTOR);

short gsp_seg_to_vect(DESCRIPTOR,short,short,short,short,DESCRIPTOR);
short gsp_vect_to_seg(DESCRIPTOR,DESCRIPTOR,short,short,short,short);
short gsp_trans_vect(DESCRIPTOR,DESCRIPTOR,long);
short gsp_extrem_vect(DESCRIPTOR,DESCRIPTOR,long,long);
short gsp_convol_vect(DESCRIPTOR,DESCRIPTOR,DESCRIPTOR);
short gsp_vect_transfer(DESCRIPTOR,DESCRIPTOR,unsigned short);
short gsp_vect_data_shift(DESCRIPTOR,short);
short gsp_vect_1c2(unsigned short,DESCRIPTOR,long,DESCRIPTOR);

short gsp_contour(DESCRIPTOR,short,short,short,unsigned short,DESCRIPTOR);
short gsp_trans_scan(DESCRIPTOR,short,short,short,short,short,unsigned short,unsigned short,DESCRIPTOR);

short gsp_code(DESCRIPTOR,unsigned short,unsigned short,DESCRIPTOR);
short gsp_process(DESCRIPTOR,unsigned short,unsigned short,DESCRIPTOR);
short gsp_dyn_code(DESCRIPTOR,DESCRIPTOR,DESCRIPTOR,DESCRIPTOR);
short gsp_dyn_process(DESCRIPTOR,DESCRIPTOR,DESCRIPTOR,DESCRIPTOR);

short bic_code(DESCRIPTOR,unsigned short,unsigned short,DESCRIPTOR);
short bic_process(DESCRIPTOR,unsigned short,unsigned short,DESCRIPTOR);
short bic_dyn_code(DESCRIPTOR,DESCRIPTOR,DESCRIPTOR,DESCRIPTOR);
short bic_dyn_process(DESCRIPTOR,DESCRIPTOR,DESCRIPTOR,DESCRIPTOR);

short gsp_reconstruct(DESCRIPTOR);
short gsp_analyse(DESCRIPTOR,unsigned long);
short gsp_feature_select(DESCRIPTOR,unsigned long,unsigned long,unsigned long,unsigned short);
short gsp_location_select(DESCRIPTOR,DESCRIPTOR,unsigned short);
short gsp_sort_object(DESCRIPTOR,unsigned long,unsigned short);
short gsp_draw_object(DESCRIPTOR,DESCRIPTOR,unsigned short);
short gsp_draw_object_using_pen(DESCRIPTOR,DESCRIPTOR,DESCRIPTOR,unsigned short,unsigned short);
short gsp_read_object(DESCRIPTOR,unsigned long,unsigned short,unsigned short,unsigned long *);
short gsp_read_all_object(DESCRIPTOR,unsigned short,unsigned short,unsigned long *);
short gsp_num_selected_segments(DESCRIPTOR,unsigned short,unsigned short,unsigned short *);
short gsp_read_segment(DESCRIPTOR,unsigned short,unsigned short,unsigned short *);
short gsp_coord2object(DESCRIPTOR,short,short,long *);
short gsp_feature_stat(DESCRIPTOR,unsigned long,long *,long *,long *);
short gsp_create_mask(DESCRIPTOR,DESCRIPTOR,short);
short gsp_associate_objects(DESCRIPTOR,DESCRIPTOR,DESCRIPTOR,DESCRIPTOR);
short gsp_detect_defects(DESCRIPTOR,DESCRIPTOR,DESCRIPTOR,long,short,long,long,short,short);

short ocr_show_character(DESCRIPTOR,DESCRIPTOR,DESCRIPTOR,short);
short ocr_get_text(DESCRIPTOR,char *);
short ocr_get_scores(DESCRIPTOR,long *,long *);
short ocr_get_positions(DESCRIPTOR,long *,long *,long *,long *);
short ocr_set_class(DESCRIPTOR,unsigned short,short,short);
short ocr_add_to_database(DESCRIPTOR,DESCRIPTOR,short,unsigned char,unsigned short);
short ocr_clear_database(DESCRIPTOR);
short ocr_read_image(DESCRIPTOR,DESCRIPTOR,DESCRIPTOR,DESCRIPTOR,short,long,char *);
short ocr_analyse_image(DESCRIPTOR,DESCRIPTOR,DESCRIPTOR,short,short);
short ocr_read_cblk(DESCRIPTOR,DESCRIPTOR,DESCRIPTOR,char *);
short ocr_analyse_cblk(DESCRIPTOR,DESCRIPTOR);

short gsp_send_msg(unsigned short,const void *,unsigned short);
short gsp_receive_msg(unsigned short,unsigned short *,void *,unsigned short);

short eva_lut(DESCRIPTOR);
short eva_lut_fill(unsigned short);
short eva_lut_transform(long,unsigned short);
short eva_profile(DESCRIPTOR);
short eva_profile_fill(unsigned short);
short eva_profile_mode(short,int);
short eva_set_connect(unsigned short);

short mesu_measure_point(DESCRIPTOR,DESCRIPTOR,short *,long *);
short mesu_measure_point_xy(DESCRIPTOR,DESCRIPTOR,short *,long *,long *);
short mesu_measure_projection(DESCRIPTOR,DESCRIPTOR,short,short *,long *);
short mesu_measure_projection_xy(DESCRIPTOR,DESCRIPTOR,short,short *,long *,long *);
short mesu_evaluate_point_count(DESCRIPTOR);
short mesu_select_point(DESCRIPTOR,long);
short mesu_measure_outline(DESCRIPTOR,DESCRIPTOR,DESCRIPTOR);

short jpeg_copy(DESCRIPTOR,DESCRIPTOR);
short jpeg_compress(DESCRIPTOR,DESCRIPTOR);
short jpeg_decompress(DESCRIPTOR,DESCRIPTOR);

short com_send_char(short,char);
short com_receive_char(short,char *);
short com_send_data(short,int,char *);
short com_receive_data(short,int *,char *);
short com_send_vect(short,DESCRIPTOR);
short com_receive_vect(short,DESCRIPTOR);
short gsp_receive_download(short,unsigned long);
short gsp_receive_download_1(short,unsigned long,short);
short keyboard_init(short,short,short *);
short keyboard_getch(short *);
short mouse_init(short,short,short,short,short,short,short,short,short);
short mouse_reset(short,short,short,short,short,short,short);
short mouse_get_state(short *,short *,short *);
short mouse_get_click(short *,short *,short *,short *,short *,short *);


short gsp_hide_desc(DESCRIPTOR,unsigned short);
short gsp_wait_event(unsigned short);
short gsp_set_expert_mode(unsigned short);

void *eure_calloc();
void *eure_malloc();
void *eure_realloc();
void  eure_free();

short IsCrisplantSolo();

#endif


#ifdef __cplusplus
}
#endif

#endif
